@@ -17,7 +17,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 
-#include <fox-1.2/fx.h>
+#include <fox-1.4/fx.h>
 
 #include "CommandWindow.h"
 #include "MessageBox.h"
@@ -40,7 +40,7 @@ FXIMPLEMENT(CommandWindow,FXDialogBox,CommandWindowMap,ARRAYNUMBER(CommandWindow
 
 // Construct window which will always float over the owner window
 CommandWindow::CommandWindow(FXWindow *owner, const FXString& name, FXString strcmd, FXint nblines, FXint nbcols):
-        FXDialogBox(owner,name,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0, 6,6,6,6, 4,4)
+        FXDialogBox(owner,name,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_MAXIMIZE,0,0,0,0, 6,6,6,6, 4,4)
 {
 
     // Get command to execute
@@ -62,7 +62,7 @@ CommandWindow::CommandWindow(FXWindow *owner, const FXString& name, FXString str
 
 // Construct free-floating window
 CommandWindow::CommandWindow(FXApp* a, const FXString& name, FXString strcmd, FXint nblines, FXint nbcols):
-        FXDialogBox(a,name,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0, 6,6,6,6, 4,4)
+        FXDialogBox(a,name,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_MAXIMIZE|DECOR_MINIMIZE,0,0,0,0, 6,6,6,6, 4,4)
 {
     // Get command to execute
     command=strcmd;
@@ -87,7 +87,6 @@ void CommandWindow::create()
     // Initialize variables
 	killed=FALSE;
 	closed=FALSE;
-    wchore = NULL;
 
     // Set text font
 	FXString fontspec;
@@ -118,7 +117,7 @@ long CommandWindow::onCmdKillProcess(FXObject*,FXSelector,void*)
 long CommandWindow::onUpdKillProcess(FXObject* sender,FXSelector,void*)
 {
     FXButton* btn = (FXButton*)sender;
-    if (!wchore)
+    if (!getApp()->hasChore(this,ID_WATCHPROCESS))
         btn->disable();
     else
         btn->enable();
@@ -129,7 +128,7 @@ long CommandWindow::onUpdKillProcess(FXObject* sender,FXSelector,void*)
 long CommandWindow::onUpdClose(FXObject* sender,FXSelector,void*)
 {
     FXButton* btn = (FXButton*)sender;
-    if (!wchore)
+    if (!getApp()->hasChore(this,ID_WATCHPROCESS))
         btn->enable();
     else
         btn->disable();
@@ -172,8 +171,8 @@ FXint CommandWindow::execCmd(FXString command)
     {
         // Here, we are running as the parent process!
         // Make sure we get called so we can check when child has finished
-        wchore = getApp()->addChore(this,ID_WATCHPROCESS);
-    }
+         getApp()->addChore(this,ID_WATCHPROCESS);
+   }
     return 0;
 }
 
@@ -188,7 +187,6 @@ long CommandWindow::onWatchProcess(FXObject*,FXSelector,void*)
 	{
         // The close button was pressed : just close the pipes
 		// and delete the object
-		wchore = NULL;
         
 		// Close pipes
 		::close(pipes[0]);
@@ -201,7 +199,7 @@ long CommandWindow::onWatchProcess(FXObject*,FXSelector,void*)
 	else if ( (waitpid(pid,NULL,WNOHANG)==0 ) )
     {
         // Child is still running, just wait
-        wchore = getApp()->addChore(this,ID_WATCHPROCESS);
+        getApp()->addChore(this,ID_WATCHPROCESS);
 
 		// Read data from the running child (first, set I-O to non-blocking)
         FXint pflags;
@@ -226,7 +224,6 @@ long CommandWindow::onWatchProcess(FXObject*,FXSelector,void*)
 	else
     {
         // Child has finished.
-        wchore = NULL;
 
 		// Read data from the finished child
 		while ((nread = read(pipes[0], buf, sizeof(buf)-1)) > 0 )
@@ -294,8 +291,7 @@ FXint CommandWindow::getLength(void)
 // Clean up
 CommandWindow::~CommandWindow()
 {
-    if (wchore)
-        wchore = getApp()->removeChore(wchore);
+	getApp()->removeChore(this,ID_WATCHPROCESS);
 
     text=(FXText*)-1;
 }
