@@ -3,10 +3,9 @@
 
 struct FileAssoc;
 class  FileDict;
-class  DirItem;
+//class  DirItem;
 class  DirList;
 
-
 // Tree item
 class FXAPI TreeItem : public FXTreeItem
 {
@@ -27,14 +26,13 @@ class FXAPI DirItem : public FXTreeItem
     FXDECLARE(DirItem)
     friend class DirList;
 protected:
-    DirItem    *iprev;
-    DirItem    *inext;
-    DirItem    *list;
-    FileAssoc  *assoc;
-    unsigned long size;
-    FXTime        date;
+    FileAssoc  *assoc;              // File association
+    DirItem    *link;               // Link to next item
+    DirItem    *list;               // List of child items
+    FXlong      size;               // File size (if a file)
+    FXTime      date;               // Time of item
 protected:
-    DirItem():iprev(NULL),inext(NULL),list(NULL),assoc(NULL),date(0)
+    DirItem():assoc(NULL),link(NULL),list(NULL),size(0L),date(0)
     {}
 protected:
     enum {
@@ -48,8 +46,10 @@ protected:
     };
 public:
     // Constructor
-    DirItem(const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL):FXTreeItem(text,oi,ci,ptr),iprev(NULL),inext(NULL),list(NULL),assoc(NULL),size(0),date(0)
-    {}
+    DirItem(const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL):FXTreeItem(text,oi,ci,ptr),assoc(NULL),link(NULL),list(NULL),size(0),date(0)
+    {
+        state=HASITEMS;
+    }
     FXbool isDirectory() const
     {
         return (state&FOLDER)!=0;
@@ -98,16 +98,14 @@ class FXAPI DirList : public FXTreeList
 {
     FXDECLARE(DirList)
 protected:
-	TreeItem 		*prevSelItem;
+    TreeItem 		*prevSelItem;
+    DirItem         *list;                  // Root item list
     FileDict		*associations;			// Association table
     FXString		dropdirectory;			// Drop directory
     FXDragAction	dropaction;				// Drop action
     FXString		dragfiles;				// Dragged files
     FXString		pattern;				// Pattern of file names
     FXuint			matchmode;				// File wildcard match mode
-    FXTimer			*refreshtimer;			// Refresh timer
-    FXTimer			*mountstimer;           // Mounts timer
-    FXTimer			*expandtimer;           // Expand folder when hovering
     FXuint			counter;				// Refresh counter
     FXIcon			*closed_folder;			// Closed folder icon
     FXIcon			*open_folder;			// Open folder icon
@@ -119,23 +117,23 @@ protected:
     FXIcon			*cdromicon;
     FXIcon			*zipicon;
     FXIcon			*linkicon;
-	FXIcon			*locked_folder;
+    FXIcon			*locked_folder;
 protected:
     DirList();
     virtual TreeItem* createItem(const FXString& text,FXIcon* oi,FXIcon* ci,void* ptr);
-    FXchar *getpath(const TreeItem* item,FXchar* pathname) const;
     TreeItem* getitem(FXchar* pathname);
-    FXbool scanRootDir(FXbool relist);
-    FXbool scanSubDir(DirItem *par,FXchar *pathname,FXbool relist);
-    FXbool listSubDir(DirItem *par,FXchar *pathname);
-    FXbool listRoots();
+    void listRootItems();
+    void listChildItems(DirItem *par);
 private:
     DirList(const DirList&);
     DirList &operator=(const DirList&);
 public:
-	long onRefresh(FXObject*,FXSelector,void*);
+    long onRefresh(FXObject*,FXSelector,void*);
     long onRefreshTimer(FXObject*,FXSelector,void*);
-	long onMountsTimer(FXObject*,FXSelector,void*);
+#if defined(linux)
+    long onMtdevicesRefresh(FXObject*,FXSelector,void*);
+    long onUpdevicesRefresh(FXObject*,FXSelector,void*);
+#endif
     long onExpandTimer(FXObject*,FXSelector,void*);
     long onBeginDrag(FXObject*,FXSelector,void*);
     long onEndDrag(FXObject*,FXSelector,void*);
@@ -168,12 +166,16 @@ public:
     long onUpdSetPattern(FXObject*,FXSelector,void*);
     long onCmdSortReverse(FXObject*,FXSelector,void*);
     long onUpdSortReverse(FXObject*,FXSelector,void*);
+    long onCmdSortCase(FXObject*,FXSelector,void*);
+    long onUpdSortCase(FXObject*,FXSelector,void*);
 public:
-    static FXint cmpFName(const TreeItem* a,const TreeItem* b);
-    static FXint cmpRName(const TreeItem* pa,const TreeItem* pb);
+    static FXint ascending(const TreeItem* a,const TreeItem* b);
+    static FXint descending(const TreeItem* a,const TreeItem* b);
+    static FXint ascendingCase(const TreeItem* a,const TreeItem* b);
+    static FXint descendingCase(const TreeItem* a,const TreeItem* b);
 public:
     enum {
-        ID_REFRESHTIMER=FXTreeList::ID_LAST,
+        ID_REFRESH_TIMER=FXTreeList::ID_LAST,
         ID_SHOW_FILES,
         ID_HIDE_FILES,
         ID_TOGGLE_FILES,
@@ -182,8 +184,11 @@ public:
         ID_TOGGLE_HIDDEN,
         ID_SET_PATTERN,
         ID_SORT_REVERSE,
-        ID_EXPANDTIMER,
-		ID_MOUNTSTIMER,
+        ID_EXPAND_TIMER,
+#if defined(linux)
+        ID_UPDEVICES_REFRESH,
+        ID_MTDEVICES_REFRESH,
+#endif
         ID_LAST
     };
 public:
@@ -194,6 +199,9 @@ public:
     // Create server-side resources
     virtual void create();
 
+    // Scan the directories and update the items if needed, or if force is TRUE
+    void scan(FXbool force=TRUE);
+
     // Return TRUE if item is a directory
     FXbool isItemDirectory(const TreeItem* item) const;
 
@@ -203,14 +211,20 @@ public:
     // Return TRUE if item is executable
     FXbool isItemExecutable(const TreeItem* item) const;
 
+    // Collapse tree
+    virtual FXbool collapseTree(TreeItem* tree,FXbool notify=FALSE);
+
+    // Expand tree
+    virtual FXbool expandTree(TreeItem* tree,FXbool notify=FALSE);
+
     // Set current file
-    void setCurrentFile(const FXString& file);
+    void setCurrentFile(const FXString& file,FXbool notify=FALSE);
 
     // Return current file
     FXString getCurrentFile() const;
 
     // Set current directory
-    void setDirectory(const FXString& path);
+    void setDirectory(const FXString& pathname,FXbool notify);
 
     // Return current directory
     FXString getDirectory() const;
@@ -218,9 +232,12 @@ public:
     // Return name of item
     FXString getItemFilename(const TreeItem* item) const;
 
-    // Return full pathname of item
+    /// Return absolute pathname of item
     FXString getItemPathname(const TreeItem* item) const;
 
+    /// Return the item from the absolute pathname
+    TreeItem* getPathnameItem(const FXString& path);
+
     // Change wildcard matching pattern
     void setPattern(const FXString& ptrn);
 
