@@ -8,8 +8,9 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include <fox-1.2/fx.h>
+#include <fox-1.4/fx.h>
 
+#include "File.h"
 #include "DirList.h"
 #include "DirListBox.h"
 
@@ -24,7 +25,7 @@ FXDEFMAP(DirListBox) DirListBoxMap[]={
 FXIMPLEMENT(DirListBox,FXDialogBox,DirListBoxMap,ARRAYNUMBER(DirListBoxMap))
 
 // Make some windows
-DirListBox::DirListBox(FXWindow *win): FXDialogBox(win,_("Directory List"), DECOR_TITLE|DECOR_BORDER,0,0,420,480)
+DirListBox::DirListBox(FXWindow *win): FXDialogBox(win,_("Directory List"),DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_MAXIMIZE,0,0,420,480)
 {
     FXVerticalFrame *contents=new FXVerticalFrame(this,LAYOUT_SIDE_TOP|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
 
@@ -50,7 +51,7 @@ void DirListBox::create()
 
 void DirListBox::setPath(const FXString &path)
 {
-    dirtree->setDirectory(path);
+    dirtree->setDirectory(path,TRUE);
 }
 
 FXString DirListBox::getPath()
@@ -59,11 +60,11 @@ FXString DirListBox::getPath()
     item = (TreeItem*)dirtree->getCurrentItem();
     if (item == NULL)
         return "";
-    FXString thepath;
-    thepath = dirtree->getItemPathname(item);
-    if(access(thepath.text(),R_OK|X_OK))
+    FXString path;
+    path = dirtree->getItemPathname(item);
+    if (!::isReadExecutable(path))
         return "";
-    return thepath;
+    return path;
 }
 
 FXuint DirListBox::execute()
@@ -71,7 +72,7 @@ FXuint DirListBox::execute()
     FXint rx,ry;
     rx = ry = 0;
     show(PLACEMENT_OWNER);
-    dirtree->expandTree(dirtree->getFirstItem());
+    dirtree->expandTree((TreeItem*)dirtree->getFirstItem());
     return getApp()->runModalFor(this);
 }
 
