@@ -2,8 +2,6 @@
 #define DIRLISTBOX_H
 
 class DirList;
-class FXDialog;
-class FXApp;
 
 class DirListBox : public FXDialogBox
 {
@@ -15,21 +13,33 @@ private:
 
 protected:
     DirList *dirtree;
-    FXTextField *directory;
 
 public:
 
 public:
     enum{
         ID_DIRECTORY=FXDialogBox::ID_LAST,
+		ID_DIRLIST_BOX,
         ID_LAST
     };
 
 public:
+    
+	// Construct a directory box list
+	DirListBox(FXWindow *win);
+
+    // Create server-side resources
+    virtual void create();
+
+	// Destructor
+    virtual ~DirListBox();
+
+    FXuint execute();
+	long onExpand(FXObject*,FXSelector,void* ptr);
     FXString getPath();
     void setPath(const FXString &path);
-    DirListBox(FXWindow *win);
-    FXuint execute();
+
+    
 };
 
 #endif
