@@ -6,10 +6,11 @@
 #include <unistd.h>
 #include <sys/wait.h>
 
-#include <fox-1.2/fx.h>
-#include <fox-1.2/fxkeys.h>
-#include <fox-1.2/FXPNGIcon.h>
+#include <fox-1.4/fx.h>
+#include <fox-1.4/fxkeys.h>
+#include <fox-1.4/FXPNGIcon.h>
 
+#include "xfedefs.h"
 #include "icons.h"
 #include "DirPanel.h"
 #include "DirList.h"
@@ -20,35 +21,19 @@
 #include "MessageBox.h"
 
 
-// Root directory string
-#ifndef ROOTDIR
-#define ROOTDIR "/"
-#endif
-
-// History size
-#ifndef ADD_TO_ARCH_HIST_SIZE
-#define ADD_TO_ARCH_HIST_SIZE 30
-#endif
-
-// Maximum length of a file name
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 1024
-#endif
-
 // Global variables
 extern FXbool confirm_del;
 extern FXString clipboard;
 extern FXuint clptype;
 extern FXMainWindow *mainWindow;
 #if defined(linux)
-extern FXStringDict* devices;
+extern FXStringDict* fsdevices;
+extern FXStringDict* mtdevices;
 #endif
 
 // Map
 FXDEFMAP(DirPanel) DirPanelMap[]={
-                                   FXMAPFUNC(SEL_COMMAND,DirPanel::ID_EXPANDDIR,DirPanel::onExpandDir),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_EXPANDTREE,DirPanel::onExpandTree),
-                                   FXMAPFUNC(SEL_COMMAND,DirPanel::ID_COLLAPSEDIR,DirPanel::onCollapseDir),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_COLLAPSETREE,DirPanel::onCollapseTree),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_PROPERTIES,DirPanel::onCmdProperties),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_ARCHIVE,DirPanel::onCmdAddToArch),
@@ -56,25 +41,35 @@ FXDEFMAP(DirPanel) DirPanelMap[]={
                                	   FXMAPFUNC(SEL_CLICKED,DirPanel::ID_DIR_PANEL,DirPanel::onCmdDirectory),
                                	   FXMAPFUNC(SEL_EXPANDED,DirPanel::ID_DIR_PANEL,DirPanel::onExpand),
                                	   FXMAPFUNC(SEL_COMMAND,DirPanel::ID_DIR_COPY,DirPanel::onCmdDirMan),
+                                   FXMAPFUNC(SEL_COMMAND,DirPanel::ID_DIR_COPYTO,DirPanel::onCmdDirMan),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_DIR_MOVE,DirPanel::onCmdDirMan),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_DIR_RENAME,DirPanel::onCmdDirMan),
-                                   FXMAPFUNC(SEL_COMMAND,DirPanel::ID_DIR_SLINK,DirPanel::onCmdDirMan),
+                                   FXMAPFUNC(SEL_COMMAND,DirPanel::ID_DIR_SYMLINK,DirPanel::onCmdDirMan),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_DIR_DELETE,DirPanel::onCmdDirDel),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_NEW_DIR,DirPanel::onCmdNewDir),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_XTERM,DirPanel::onCmdXTerm),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_COPY_CLP,DirPanel::onCmdClp),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_CUT_CLP,DirPanel::onCmdClp),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_PASTE_CLP,DirPanel::onCmdPaste),
+                                   FXMAPFUNC(SEL_COMMAND,DirPanel::ID_SORT_REVERSE,DirPanel::onCmdSortReverse),
+                                   FXMAPFUNC(SEL_COMMAND,DirPanel::ID_TOGGLE_HIDDEN,DirPanel::onCmdToggleHidden),
+                                   FXMAPFUNC(SEL_COMMAND,DirPanel::ID_TOGGLE_TREE,DirPanel::onCmdToggleTree),
 #if defined(linux)
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_MOUNT,DirPanel::onCmdMount),
                                    FXMAPFUNC(SEL_COMMAND,DirPanel::ID_UMOUNT,DirPanel::onCmdMount),
+                               	   FXMAPFUNC(SEL_UPDATE,DirPanel::ID_MOUNT,DirPanel::onUpdMount),
+                                   FXMAPFUNC(SEL_UPDATE,DirPanel::ID_UMOUNT,DirPanel::onUpdUnmount),
 #endif								   
                                	   FXMAPFUNC(SEL_UPDATE,DirPanel::ID_PASTE_CLP,DirPanel::onUpdPaste),
                                    FXMAPFUNC(SEL_UPDATE,DirPanel::ID_TOGGLE_HIDDEN,DirPanel::onUpdToggleHidden),
-                                   FXMAPFUNC(SEL_COMMAND,DirPanel::ID_TOGGLE_HIDDEN,DirPanel::onCmdToggleHidden),
-                                   FXMAPFUNC(SEL_COMMAND,DirPanel::ID_TOGGLE_TREE,DirPanel::onCmdToggleTree),
                                    FXMAPFUNC(SEL_UPDATE,DirPanel::ID_TOGGLE_TREE,DirPanel::onUpdToggleTree),
-                                   FXMAPFUNC(SEL_UPDATE,DirPanel::ID_STATUS,DirPanel::onUpdStatus),
+                                   FXMAPFUNC(SEL_UPDATE,DirPanel::ID_CUT_CLP,DirPanel::onUpdMenu),
+                                   FXMAPFUNC(SEL_UPDATE,DirPanel::ID_ARCHIVE,DirPanel::onUpdMenu),
+                                   FXMAPFUNC(SEL_UPDATE,DirPanel::ID_DIR_MOVE,DirPanel::onUpdMenu),
+                                   FXMAPFUNC(SEL_UPDATE,DirPanel::ID_DIR_RENAME,DirPanel::onUpdMenu),
+                                   FXMAPFUNC(SEL_UPDATE,DirPanel::ID_DIR_DELETE,DirPanel::onUpdMenu),
+                                   FXMAPFUNC(SEL_UPDATE,DirPanel::ID_NEW_DIR,DirPanel::onUpdMenu),
+                                    FXMAPFUNC(SEL_UPDATE,DirPanel::ID_STATUS,DirPanel::onUpdStatus),
                           		};
 
 // Object implementation 
@@ -87,6 +82,7 @@ DirPanel::DirPanel(FXComposite *p,FXuint opts,FXint x,FXint y,FXint w,FXint h):
 	// Create some icons
 	attricon=new FXPNGIcon(getApp(),attrib);
     archicon=new FXPNGIcon(getApp(),arch);
+    sorticon=new FXPNGIcon(getApp(),sort);
     newfoldericon=new FXPNGIcon(getApp(),newfolder);
 	delicon=new FXPNGIcon(getApp(),filedelete);	
 	copyicon=new FXPNGIcon(getApp(),copy_clp);	
@@ -107,10 +103,8 @@ DirPanel::DirPanel(FXComposite *p,FXuint opts,FXint x,FXint y,FXint w,FXint h):
 	FXIcon *hidehiddenicon=new FXPNGIcon(getApp(),hidehidden);
 
 	// Construct directory panel
-    FXVerticalFrame* cont=new FXVerticalFrame(this,LAYOUT_FILL_Y|LAYOUT_FILL_X|FRAME_SUNKEN,0,0,0,0, 0,0,0,0, 1,1);    
-
+    FXVerticalFrame* cont=new FXVerticalFrame(this,LAYOUT_FILL_Y|LAYOUT_FILL_X|FRAME_SUNKEN,0,0,0,0, 0,0,0,0, 1,1);
 	FXPacker* packer=new FXHorizontalFrame(cont,JUSTIFY_LEFT|LAYOUT_FILL_X|FRAME_RAISED,0,0,0,0, 0,0,0,0);
-    new FXButton(packer,_("\tHide"),new FXPNGIcon(getApp(),small_x),this,FXWindow::ID_HIDE,BUTTON_TOOLBAR|LAYOUT_RIGHT|FRAME_RAISED,0,0,0,0, 3,3,0,0);  
 	new FXLabel(packer,_("Folders"),NULL,JUSTIFY_LEFT|LAYOUT_LEFT|LAYOUT_FILL_X);
 
     list=new DirList(cont,this,ID_DIR_PANEL,LAYOUT_FILL_X|LAYOUT_FILL_Y|TREELIST_BROWSESELECT|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|ICONLIST_BIG_ICONS);
@@ -118,7 +112,7 @@ DirPanel::DirPanel(FXComposite *p,FXuint opts,FXint x,FXint y,FXint w,FXint h):
 	statusbar=new FXHorizontalFrame(cont,JUSTIFY_LEFT|LAYOUT_FILL_X,0,0,0,0, 3,3,3,3);
     statusbar->setTarget(this);
     statusbar->setSelector(FXSEL(SEL_UPDATE,DirPanel::ID_STATUS));
-    new FXToggleButton(statusbar,_("\tShow hidden folders (Ctrl-F6)"),_("\tHide hidden folders (Ctrl-F6)"),showhiddenicon,hidehiddenicon,this,
+    new FXToggleButton(statusbar,_("\tShow hidden folders (Ctrl-F5)"),_("\tHide hidden folders (Ctrl-F5)"),showhiddenicon,hidehiddenicon,this,
 	                   ID_TOGGLE_HIDDEN,BUTTON_TOOLBAR|LAYOUT_LEFT|ICON_BEFORE_TEXT|FRAME_RAISED);
     status=new FXLabel(statusbar,_("Status"),NULL,JUSTIFY_LEFT|LAYOUT_LEFT|LAYOUT_FILL_X|FRAME_SUNKEN);
 }
@@ -129,6 +123,7 @@ DirPanel::~DirPanel()
 {
 	delete attricon;
 	delete archicon;
+	delete sorticon;
 	delete newfoldericon;
 	delete delicon;
 	delete copyicon;
@@ -169,8 +164,8 @@ long DirPanel::onCmdDirectory(FXObject*,FXSelector,void* ptr)
     TreeItem *item=(TreeItem*)ptr;
     if(item)
     {
-        FXString directory = list->getItemPathname(item);
-        if(access(directory.text(),R_OK|X_OK))
+        FXString directory=list->getItemPathname(item);
+        if(!::isReadExecutable(directory))
         {
             MessageBox::error(this,MBOX_OK,_("Error"),_(" Permission to: %s denied."), directory.text());
             return 0;
@@ -192,8 +187,8 @@ long DirPanel::onCmdDirectory(FXObject*,FXSelector,void* ptr)
 		}
        	while(i < count)
 		{
-       		item = address->getItem(i++);
-       		if(!strcmp((const char*)&p[0],(const char*)&item[0]))
+       		item=address->getItem(i++);
+       		if(::streq((const char*)&p[0],(const char*)&item[0]))
 			{
 				i--;
 				break;
@@ -239,7 +234,7 @@ long DirPanel::onUpdToggleTree(FXObject* sender,FXSelector sel,void* ptr)
 }
 
 
-// File list context menu
+// Directory list context menu
 long DirPanel::onPopupMenu(FXObject* o,FXSelector s,void* p)
 {
 	// Current item becomes item under cursor
@@ -255,44 +250,43 @@ long DirPanel::onPopupMenu(FXObject* o,FXSelector s,void* p)
 	// If item, then set it current and set directory in DirList and FileList
 	list->setCurrentItem(item,TRUE);
 	FXString dir=list->getItemPathname((TreeItem*)item);
-	list->setDirectory(dir);
+	list->setDirectory(dir,TRUE);
 	((XFileExplorer*) mainWindow)->lpanel->current->list->setDirectory(dir);
 
     // Popup menu pane
     FXMenuPane menu(this);
     
 	// Menu
-    new FXMenuCommand(&menu,_("&Terminal"),xtermicon,this,ID_XTERM);
-	new FXMenuCommand(&menu,_("New f&older..."),newfoldericon,this,ID_NEW_DIR);
-	new FXMenuCommand(&menu,_("&Add to archive..."),archicon,this,ID_ARCHIVE);
 #if defined(linux)
-	if (isLink(dir))
+	if (::isLink(dir))
 		dir=readlink(dir);
-    if(devices->find(dir.text()))
+    if(fsdevices->find(dir.text()))
 	{
-    	new FXMenuSeparator(&menu);
 		new FXMenuCommand(&menu,_("&Mount"),mapicon,this,ID_MOUNT);
 		new FXMenuCommand(&menu,_("Unmount"),unmapicon,this,ID_UMOUNT);
+    	new FXMenuSeparator(&menu);
 	}
 #endif
+    new FXMenuCommand(&menu,_("&Terminal"),xtermicon,this,DirPanel::ID_XTERM);
+	new FXMenuCommand(&menu,_("New f&older..."),newfoldericon,this,DirPanel::ID_NEW_DIR);
+	new FXMenuCommand(&menu,_("&Add to archive..."),archicon,this,DirPanel::ID_ARCHIVE);
     new FXMenuSeparator(&menu);
-	new FXMenuCommand(&menu,_("&Copy"),copyicon,this,ID_COPY_CLP);
-	new FXMenuCommand(&menu,_("C&ut"),cuticon,this,ID_CUT_CLP);
-	if(!clipboard.empty())
-		new FXMenuCommand(&menu,_("&Paste"),pasteicon,this,ID_PASTE_CLP);
+	new FXMenuCommand(&menu,_("&Copy"),copyicon,this,DirPanel::ID_COPY_CLP);
+	new FXMenuCommand(&menu,_("C&ut"),cuticon,this,DirPanel::ID_CUT_CLP);
+	new FXMenuCommand(&menu,_("&Paste"),pasteicon,this,DirPanel::ID_PASTE_CLP);
     new FXMenuSeparator(&menu);
-	new FXMenuCommand(&menu,_("&Move..."),moveicon,this,ID_DIR_MOVE);
-	new FXMenuCommand(&menu,_("Re&name..."),renameicon,this,ID_DIR_RENAME);
-	new FXMenuCommand(&menu,_("Symlin&k..."),symlinkicon,this,ID_DIR_SLINK);
-	new FXMenuCommand(&menu,_("&Delete"),delicon,this,ID_DIR_DELETE);
+	new FXMenuCommand(&menu,_("Re&name..."),renameicon,this,DirPanel::ID_DIR_RENAME);
+	new FXMenuCommand(&menu,_("Copy &to..."),copyicon,this,DirPanel::ID_DIR_COPYTO);
+	new FXMenuCommand(&menu,_("&Move..."),moveicon,this,DirPanel::ID_DIR_MOVE);
+	new FXMenuCommand(&menu,_("Symlin&k..."),symlinkicon,this,DirPanel::ID_DIR_SYMLINK);
+	new FXMenuCommand(&menu,_("&Delete"),delicon,this,DirPanel::ID_DIR_DELETE);
     new FXMenuSeparator(&menu);
-	new FXMenuCheck(&menu,_("&Hidden folders"),this,ID_TOGGLE_HIDDEN);
-	new FXMenuCommand(&menu,_("&Expand"),expfoldericon,this,ID_EXPANDDIR);
-	new FXMenuCommand(&menu,_("Co&llapse"),collfoldericon,this,ID_COLLAPSEDIR);
-	new FXMenuCommand(&menu,_("E&xpand tree"),exptreeicon,this,ID_EXPANDTREE);
-	new FXMenuCommand(&menu,_("Collap&se tree"),colltreeicon,this,ID_COLLAPSETREE);
+	new FXMenuCheck(&menu,_("&Hidden folders"),this,DirPanel::ID_TOGGLE_HIDDEN);
+	new FXMenuCommand(&menu,_("&Reverse order"),sorticon,this,DirPanel::ID_SORT_REVERSE);
+	new FXMenuCommand(&menu,_("E&xpand tree"),exptreeicon,this,DirPanel::ID_EXPANDTREE);
+	new FXMenuCommand(&menu,_("Collap&se tree"),colltreeicon,this,DirPanel::ID_COLLAPSETREE);
     new FXMenuSeparator(&menu);
-	new FXMenuCommand(&menu,_("Proper&ties"),attricon,this,ID_PROPERTIES);
+	new FXMenuCommand(&menu,_("Proper&ties"),attricon,this,DirPanel::ID_PROPERTIES);
 	
     menu.create();
     menu.popup(NULL,x,y);
@@ -301,17 +295,6 @@ long DirPanel::onPopupMenu(FXObject* o,FXSelector s,void* p)
 }
 
 
-// Expand a directory in the tree
-long DirPanel::onExpandDir(FXObject* sender,FXSelector sel,void* ptr)
-{
-    // Current item
-	DirItem* item=(DirItem*)list->getCurrentItem();
-
-    // Expand tree item
-    list->expandTree(item,TRUE);
-
-	return 1;
-}
 
 
 // Helper function used to explore the directory tree and expand or collapse it
@@ -320,9 +303,9 @@ long DirPanel::exploreUp(DirItem *item, const DirItem *rootitem, const FXint tas
 	DirItem* parentitem=item;
 	
 	if (task==ID_EXPANDTREE)
-		list->expandTree(item,TRUE);
+		list->expandTree((TreeItem*)item,TRUE);
 	else
-		list->collapseTree(item,TRUE);
+		list->collapseTree((TreeItem*)item,TRUE);
 	
 	item=(DirItem*)item->getFirst();
 	if (!item)
@@ -342,9 +325,9 @@ long DirPanel::exploreDown(DirItem *item, const DirItem *rootitem, const FXint t
 	DirItem* parentitem=(DirItem*)item->getParent();
 	
 	if (task==ID_EXPANDTREE)
-		list->expandTree(item,TRUE);
+		list->expandTree((TreeItem*)item,TRUE);
 	else
-		list->collapseTree(item,TRUE);
+		list->collapseTree((TreeItem*)item,TRUE);
 	item=(DirItem*)item->getNext();
 	
 	if (!item)
@@ -352,9 +335,9 @@ long DirPanel::exploreDown(DirItem *item, const DirItem *rootitem, const FXint t
 	else
 	{
 		if (task==ID_EXPANDTREE)
-			list->expandTree(item,TRUE);
+			list->expandTree((TreeItem*)item,TRUE);
 		else
-			list->collapseTree(item,TRUE);
+			list->collapseTree((TreeItem*)item,TRUE);
 		if (!list->isItemLeaf(item))
 			exploreUp(item,rootitem,task);
 		else
@@ -388,18 +371,6 @@ long DirPanel::onCollapseTree(FXObject* sender,FXSelector sel,void* ptr)
 }
 
 
-// Collapse a directory in the tree
-long DirPanel::onCollapseDir(FXObject* sender,FXSelector sel,void* ptr)
-{
-	// Current item
-    DirItem* item=(DirItem*)list->getCurrentItem();
-
-    // Collapse tree item
-    list->collapseTree(item,TRUE);
-	
-	return 1;
-}
-
 // Directory properties
 long DirPanel::onCmdProperties(FXObject* sender,FXSelector,void* ptr)
 {
@@ -407,9 +378,9 @@ long DirPanel::onCmdProperties(FXObject* sender,FXSelector,void* ptr)
 	DirItem* item=(DirItem*)list->getCurrentItem();
 	FXString pathname=list->getItemPathname((TreeItem*)item);
 	
-	PropertiesBox* attrdlg = new PropertiesBox(this,FXFile::name(pathname),FXFile::directory(pathname));	
-   	if(attrdlg->execute())
-        list->setDirectory(pathname);
+	PropertiesBox* attrdlg=new PropertiesBox(this,FXFile::name(pathname),FXFile::directory(pathname));	
+   	if(attrdlg->execute(PLACEMENT_SCREEN))
+        list->setDirectory(pathname,TRUE);
     delete attrdlg;
    	return 1;
 }
@@ -423,18 +394,17 @@ long DirPanel::onCmdAddToArch(FXObject* o,FXSelector sel,void* ptr)
 	FXString name=list->getItemText(item);
 	FXString pathname=list->getItemPathname((TreeItem*)item);	
 	FXString parentdir=FXFile::directory(pathname);
-	FXString archive="";
+	FXString archive="", cmd;
 	
     static char history[ADD_TO_ARCH_HIST_SIZE][100];
     static int n=0;
     int i;
 	File *f;	
     HistInputDialog* dialog;
-
-    // Default archive is tar.gz 
-	FXString command="/bin/tar -zcvf ";
     
-	dialog = new HistInputDialog(this,archive,_("Create new archive: (e.g. archive.tar.gz)"),_("Add To Archive"));
+	dialog=new HistInputDialog(this,archive,
+	       _("Create new archive: (e.g. archive.tar.gz)\n=>Supported formats : tar, tgz, tbz2, taz, zip, rar, lzh"),
+	       _("Add To Archive"));
     for(int i=0;i<n;i++)
         dialog->appendItem(history[i]);
     dialog->CursorEnd();
@@ -443,24 +413,29 @@ long DirPanel::onCmdAddToArch(FXObject* o,FXSelector sel,void* ptr)
         archive=parentdir+PATHSEPSTRING+dialog->getText();
 
         // Handle zip and bz2 archives
-        FXString extension=archive.rafter('.',1);
-		if (extension=="zip" || extension=="ZIP")
-            command="/usr/bin/zip -r ";
-        else if (extension=="bz2" || extension=="BZ2" || extension=="tbz2" || extension=="TBZ2")
-            command="/bin/tar -jcvf ";
-		else if (extension=="Z")
-			command="/bin/tar -Zcvf ";
+        FXString ext=archive.rafter('.',1);
+		ext.lower();
+		if (ext=="zip")
+            cmd="zip -r ";
+        else if (ext=="bz2" || ext=="tbz2")
+            cmd="tar -jcvf ";
+		else if (ext=="rar")
+			cmd="rar a ";
+		else if (ext=="lzh")
+			cmd="lha a ";
+		else if (ext=="z")
+			cmd="tar -Zcvf ";
 
 		// Archive command name
-		command+=archive+" "+"\""+name+"\"";
+		cmd+=archive+" "+"\""+name+"\"";
 					
 		// File object
-		f=new File(getApp(),_("Create archive"),ARCHIVE);
+		f=new File(this,_("Create archive"),ARCHIVE);
 		f->create();
 				   
 		// Create archive
 		chdir(parentdir.text());
-		f->archive(archive,command);
+		f->archive(archive,cmd);
  
 		// If action is cancelled in progress dialog
 		if (f->isCancelled)
@@ -473,23 +448,23 @@ long DirPanel::onCmdAddToArch(FXObject* o,FXSelector sel,void* ptr)
         // Manage dialog history size
 		n=dialog->getHistorySize();
         n++;
-        command=dialog->getText();
+        cmd=dialog->getText();
         if(n>ADD_TO_ARCH_HIST_SIZE)
             n=ADD_TO_ARCH_HIST_SIZE;
         for(i=0;i<n-1;i++)
-            if(!strcmp(history[i],command.text()))
+            if(::streq(history[i],cmd.text()))
                 break;
         if(i==n-1)
         {
-            strcpy(history[0],command.text());
+            strlcpy(history[0],cmd.text(),sizeof(history[0]));
             for(i=1;i<n;i++)
-                strcpy(history[i],dialog->getHistoryItem(i-1).text());
+                strlcpy(history[i],dialog->getHistoryItem(i-1).text(),dialog->getHistoryItem(i-1).length()+1);
         }
         else
             n--;
 
 		// Display parent directory in DirList and FileList
-		list->setDirectory(parentdir);
+		list->setDirectory(parentdir,TRUE);
 		((XFileExplorer*) mainWindow)->lpanel->current->list->setDirectory(parentdir);
     }
     delete dialog;
@@ -497,7 +472,7 @@ long DirPanel::onCmdAddToArch(FXObject* o,FXSelector sel,void* ptr)
 }
 
 
-// Copy/Move/Rename/SymLink directory
+// Copy/Move/Rename/Symlink directory
 long DirPanel::onCmdDirMan(FXObject* sender,FXSelector sel,void*)
 {
 	// Current item
@@ -506,22 +481,25 @@ long DirPanel::onCmdDirMan(FXObject* sender,FXSelector sel,void*)
 	// Parameters passed to XFileExplorer::onCmdFileMan
 	// -1 is used to tell that we are calling from DirPanel
 	FXString source=list->getItemPathname((TreeItem*)item);	
-    FXString target = FXFile::directory(source);
-	FXString param = target+"\n"+"-1"+"\n"+source;
+    FXString target=FXFile::directory(source);
+	FXString param=target+"\n"+"-1"+"\n"+source;
 
     switch(FXSELID(sel))
     {
     case ID_DIR_COPY:
-        sel = FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_COPY);
+        sel=FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_COPY);
+        break;
+    case ID_DIR_COPYTO:
+        sel=FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_COPYTO);
         break;
     case ID_DIR_MOVE:
-        sel = FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_MOVE);
+        sel=FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_MOVE);
         break;
     case ID_DIR_RENAME:
-        sel = FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_RENAME);
+        sel=FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_RENAME);
         break;
-    case ID_DIR_SLINK:
-        sel = FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_SLINK);
+    case ID_DIR_SYMLINK:
+        sel=FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_SYMLINK);
         break;
     }
     mainWindow->handle(this,sel,strdup(param.text()));
@@ -531,6 +509,13 @@ long DirPanel::onCmdDirMan(FXObject* sender,FXSelector sel,void*)
     return 1;
 }
 
+// Reverse the sort order
+long DirPanel::onCmdSortReverse(FXObject* sender,FXSelector sel,void*)
+{
+        list->handle(this,FXSEL(SEL_COMMAND,DirList::ID_SORT_REVERSE),NULL);
+		return 1;
+}
+
 
 // Delete directory
 long DirPanel::onCmdDirDel(FXObject*,FXSelector,void*)
@@ -541,7 +526,7 @@ long DirPanel::onCmdDirDel(FXObject*,FXSelector,void*)
 	FXString parentdir=FXFile::directory(pathname);	
 
     // If we don't have permission to write to the parent directory
-    if(access(parentdir.text(), W_OK))
+    if(!::isWritable(parentdir))
     {
         MessageBox::error(this,MBOX_OK,_("Error"),_("Can't write to %s: Permission denied"), parentdir.text());
         return 0;
@@ -552,16 +537,16 @@ long DirPanel::onCmdDirDel(FXObject*,FXSelector,void*)
         FXPNGIcon icon(getApp(),delete_big);
 		FXString message=_("Delete folder ") + pathname + " ?";
 		MessageBox box(this,_("Confirm Delete"),message,&icon,MBOX_OK_CANCEL|DECOR_TITLE|DECOR_BORDER);
-        if(box.execute() != MBOX_CLICKED_OK)
+        if(box.execute(PLACEMENT_CURSOR) != MBOX_CLICKED_OK)
             return 0;
     }
 
 	// File object
-   	File* f=new File(getApp(),_("File Delete"),DELETE);
+   	File* f=new File(this,_("File Delete"),DELETE);
    	f->create();
 
     // If we don't have permission to write to the directory
-    if(access(pathname.text(), W_OK) && !access(pathname.text(), F_OK))
+ 	if (!::isLWritable(pathname))
     {
 		// Get process uid and directory uid
 		FXuint uid=getuid();
@@ -578,7 +563,7 @@ long DirPanel::onCmdDirDel(FXObject*,FXSelector,void*)
 			FXPNGIcon icon(getApp(),errorIcon);
 			FXString str=_("Folder ") + pathname + _(" is write-protected, delete it anyway?");
         	MessageBox box(this,_("Confirm Delete"),str,&icon,MBOX_OK_CANCEL|DECOR_TITLE|DECOR_BORDER);        	
-			FXuint answer=box.execute();
+			FXuint answer=box.execute(PLACEMENT_SCREEN);
 			if(answer == MBOX_CLICKED_OK)
 			{
 				f->remove(pathname); 	
@@ -588,7 +573,7 @@ long DirPanel::onCmdDirDel(FXObject*,FXSelector,void*)
 					MessageBox::error(this,MBOX_OK,_("Error"),_("Delete folder operation cancelled!"));
 				
 				// Return to parent directory in DirList and FileList
-				list->setDirectory(parentdir);
+				list->setDirectory(parentdir,TRUE);
 				((XFileExplorer*) mainWindow)->lpanel->current->list->setDirectory(parentdir);
 			}
 		}
@@ -614,15 +599,15 @@ long DirPanel::onCmdDirDel(FXObject*,FXSelector,void*)
 			MessageBox::error(this,MBOX_OK,_("Error"),_("Delete file operation cancelled!"));
 		}
 		// Return to parent directory in DirList and FileList
-		list->setDirectory(parentdir);
+		list->setDirectory(parentdir,TRUE);
 		((XFileExplorer*) mainWindow)->lpanel->current->list->setDirectory(parentdir);
 	}
 
 	delete f;
 
 	// Force DirPanel and FilePanel refresh
-    list->onRefresh(0,0,0);
-	((XFileExplorer*) mainWindow)->lpanel->onRefresh(0,0,0);
+ 	//mainWindow->handle(this,FXSEL(SEL_COMMAND,XFileExplorer::ID_REFRESH),NULL);
+	list->onRefresh(0,0,0);
     return 1;
 }
 
@@ -638,9 +623,9 @@ long DirPanel::onCmdNewDir(FXObject*,FXSelector,void*)
     if(dirpath!=ROOTDIR)
         dirpath+=PATHSEPSTRING;
     
-    InputDialog* dialog = new InputDialog(this,dirname,_("Create new folder..."),_("New Folder"));
+    InputDialog* dialog=new InputDialog(this,dirname,_("Create new folder..."),_("New Folder"));
     dialog->CursorEnd();
-    if(dialog->execute())
+    if(dialog->execute(PLACEMENT_CURSOR))
     {
 		dirname=dirpath+dialog->getText();		
         if(dirname!=dirpath)
@@ -668,7 +653,7 @@ long  DirPanel::onCmdXTerm(FXObject*,FXSelector,void*)
     DirItem* item=(DirItem*)list->getCurrentItem();
 	FXString buf=list->getItemPathname((TreeItem*)item);
 	chdir(buf.text());
-    FXString cmd = getApp()->reg().readStringEntry("PROGS","xterm","xterm -sb");
+    FXString cmd=getApp()->reg().readStringEntry("PROGS","xterm","xterm -sb");
     cmd += " &";
     system(cmd.text());
     return 1;
@@ -684,19 +669,18 @@ long DirPanel::onCmdPaste(FXObject*,FXSelector sel,void*)
     FXString targetdir;
 
 	targetdir=((XFileExplorer*) mainWindow)->lpanel->current->list->getDirectory();
-    param = targetdir+"\n"+clipboard;
+    param=targetdir+"\n"+clipboard;
 
     switch(clptype)
     {
     case COPY_CLP:
-        sel = FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_COPY);
+        sel=FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_COPY);
         break;
     case CUT_CLP:
-        sel = FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_CUT);
+        sel=FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_CUT);
         break;
     }
     mainWindow->handle(this,sel,strdup(param.text()));
-    clipboard = "";
 
 	// Force dirpanel refresh
 	list->onRefresh(0,0,0);
@@ -712,15 +696,17 @@ long DirPanel::onCmdClp(FXObject*,FXSelector sel,void*)
     DirItem* item=(DirItem*)list->getCurrentItem();
 	FXString pathname=list->getItemPathname((TreeItem*)item);
 
+	// Empty clipboard
+	clipboard="";
+
 	// Clipboard type
-    clipboard = "";
 	if (FXSELID(sel)==ID_CUT_CLP)
 		clptype=CUT_CLP;
 	else
 		clptype=COPY_CLP;
 
-	clipboard += pathname+"\n";
-    clipboard = FXStringVal(1)+"\n"+clipboard;
+	clipboard+=pathname+"\n";
+    clipboard=FXStringVal(1)+"\n"+clipboard;
 
     return 1;
 }
@@ -739,7 +725,7 @@ long DirPanel::onCmdMount(FXObject*,FXSelector sel,void*)
 	FXString dir=list->getItemPathname((TreeItem*)item);
 
 	// If symbolic link
-	if (isLink(dir))
+	if (::isLink(dir))
 		dir=FXFile::symlink(dir);
 
     // Select the command and set the appropriate message
@@ -747,13 +733,13 @@ long DirPanel::onCmdMount(FXObject*,FXSelector sel,void*)
     {
         op=MOUNT;
 		msg=_("Mount");
-        cmd="/bin/mount ";
+        cmd="mount ";
     }
     else
     {
         op=UNMOUNT;
 		msg=_("Unmount");
-        cmd="/bin/umount ";
+        cmd="umount ";
     }
     cmd+=dir;
     cmd+=" 2>&1";
@@ -764,7 +750,7 @@ long DirPanel::onCmdMount(FXObject*,FXSelector sel,void*)
 
 	// File object
 	text=msg + _(" file system...");
-	f=new File(getApp(),text.text(),op);
+	f=new File(this,text.text(),op);
 	f->create();
 				   
 	// Mount/unmount file system
@@ -789,33 +775,79 @@ long DirPanel::onCmdMount(FXObject*,FXSelector sel,void*)
 
     return 1;
 }
+
+// Update the Mount menu item
+long DirPanel::onUpdMount(FXObject* o,FXSelector sel,void*)
+{
+	// Current item
+    DirItem* item=(DirItem*)list->getCurrentItem();
+	FXString dir=list->getItemPathname((TreeItem*)item);
+
+    if(fsdevices->find(dir.text()) && !mtdevices->find(dir.text()))
+        o->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
+    else
+        o->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
+
+    return 1;
+}
+
+
+// Update the Unmount menu item
+long DirPanel::onUpdUnmount(FXObject* o,FXSelector sel,void*)
+{
+	// Current item
+    DirItem* item=(DirItem*)list->getCurrentItem();
+	FXString dir=list->getItemPathname((TreeItem*)item);
+
+    if(fsdevices->find(dir.text()) && mtdevices->find(dir.text()))
+        o->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
+    else
+        o->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
+
+    return 1;
+}
 #endif
 
 
-// Is it necessary?
+// Update the paste button and menus
 long DirPanel::onUpdPaste(FXObject* o,FXSelector,void*)
 {
-    FXButton *button=(FXButton*)o;
-    int style=button->getButtonStyle();
-    if(style & BUTTON_TOOLBAR)
-    {
-        if(!clipboard.empty())
-            o->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
-        else
-            o->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
-    }
-    else
-        o->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
+	// Name of the current selected item
+	TreeItem* item=(TreeItem*)list->getCurrentItem();
+	
+	// There is no selected item
+	if (item==NULL)
+		return 0;
+	
+	// Path name of the selected item
+	FXString dir=list->getItemPathname(item);	
 
     return 1;
 }
 
 
+// Update menu items and toolbar buttons that are related to file operations
+long DirPanel::onUpdMenu(FXObject* o,FXSelector,void*)
+{
+	// Name of the current selected item
+	TreeItem* item=(TreeItem*)list->getCurrentItem();
+	
+	// There is no selected item
+	if (item==NULL)
+		return 0;
+	
+	// Path name of the selected item
+	FXString dir=list->getItemPathname(item);	
+    return 1;
+}
+
+
 // Update the status bar and the directory path name
 long DirPanel::onUpdStatus(FXObject* sender,FXSelector,void*)
 {
 	unsigned long long dsize;
-	char buf[MAXPATHLEN+1], size[64], hsize[64];
+	char buf[MAXPATHLEN+1], size[64];
+	FXString hsize;
 	
 	// Name of the current selected item
 	TreeItem* item=(TreeItem*)list->getCurrentItem();
@@ -829,25 +861,18 @@ long DirPanel::onUpdStatus(FXObject* sender,FXSelector,void*)
 	
 	// Path name of the selected item
 	FXString path=list->getItemPathname(item);	
-	
-	// File doesn't exist
-	if (!exists(dequote(path)))
-	{
-		status->setText("--");
-		return 0;
-	}
 
 	// Size of the files present in the directory
 	path=quote(path,FALSE);
-	strcpy(buf,path.text());
+	strlcpy(buf,path.text(),path.length()+1);
 	dsize=dirsize(buf);
 
 	// Size in human readable form
-	sprintf(size,"%llu",dsize);
-	::hSize(size,hsize);
+	snprintf(size,sizeof(size)-1,"%llu",dsize);
+	hsize=::hSize(size);
 
 	// Update the status label
-	sprintf(buf,_("%s"),hsize);
+	snprintf(buf,sizeof(buf)-1,_("%s"),hsize.text());
 	status->setText(buf);
 
 	// Update the path in the window title
