@@ -12,6 +12,7 @@ class DirPanel : public FXVerticalFrame
 protected:
     FXIcon	*attricon;
     FXIcon 	*archicon;
+    FXIcon 	*sorticon;
     FXIcon	*newfoldericon;
     FXIcon	*delicon;
 	FXIcon	*copyicon;
@@ -39,7 +40,6 @@ DirPanel(FXComposite *p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
     ~DirPanel();
     enum {
         ID_DIR_PANEL=FXVerticalFrame::ID_LAST,
-		ID_EXPANDDIR,
 		ID_EXPANDTREE,
 		ID_TOGGLE_HIDDEN,
 		ID_COLLAPSEDIR,
@@ -47,9 +47,10 @@ DirPanel(FXComposite *p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
 		ID_PROPERTIES,
 		ID_ARCHIVE,
 		ID_DIR_COPY,
+		ID_DIR_COPYTO,
 		ID_DIR_MOVE,
 		ID_DIR_RENAME,
-		ID_DIR_SLINK,
+		ID_DIR_SYMLINK,
 		ID_DIR_DELETE,
 		ID_NEW_DIR,
 		ID_XTERM,
@@ -58,6 +59,7 @@ DirPanel(FXComposite *p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
 		ID_PASTE_CLP,
         ID_TOGGLE_TREE,
 		ID_STATUS,
+		ID_SORT_REVERSE,
 #if defined(linux)
         ID_MOUNT,
         ID_UMOUNT,
@@ -70,9 +72,7 @@ public:
 	long onCmdToggleHidden(FXObject*,FXSelector,void*);
 	long onUpdToggleHidden(FXObject* sender,FXSelector,void*);
 	long onPopupMenu(FXObject*,FXSelector,void*);
-	long onExpandDir(FXObject*,FXSelector,void*);
 	long onExpandTree(FXObject*,FXSelector,void*);
-	long onCollapseDir(FXObject*,FXSelector,void*);
 	long onCollapseTree(FXObject*,FXSelector,void*);
 	long onCmdProperties(FXObject*,FXSelector,void*);
 	long onCmdAddToArch(FXObject*,FXSelector,void*);
@@ -86,8 +86,12 @@ public:
 	long onCmdDirectory(FXObject*,FXSelector,void* ptr);
 	long onCmdToggleTree(FXObject* sender,FXSelector sel,void* ptr);
 	long onUpdToggleTree(FXObject* sender,FXSelector sel,void* ptr);
+	long onUpdMount(FXObject* o,FXSelector sel,void*);
+	long onUpdUnmount(FXObject* o,FXSelector sel,void*);
+	long onUpdMenu(FXObject* o,FXSelector,void*);
 	long onUpdStatus(FXObject* sender,FXSelector,void*);
 	long onExpand(FXObject*,FXSelector,void* ptr);
+	long onCmdSortReverse(FXObject*,FXSelector,void* ptr);
 #if defined(linux)
 	long onCmdMount(FXObject*,FXSelector sel,void*);
 #endif
