@@ -18,9 +18,16 @@
 #include <sys/wait.h>
 #include <time.h>
 
-#include <fox-1.2/fx.h>
-#include <fox-1.2/fxkeys.h>
+// For Sun compatibility
+#ifdef __sun
+#include <alloca.h>
+#endif
 
+#include <fox-1.4/fx.h>
+#include <fox-1.4/fxkeys.h>
+#include <fox-1.4/FXPNGIcon.h>
+
+#include "xfedefs.h"
 #include "icons.h"
 #include "File.h"
 #include "OverwriteBox.h"
@@ -28,15 +35,12 @@
 #include "CommandWindow.h"
 
 #if defined(linux)
-extern FXStringDict* devices;
-extern FXStringDict* mounts;
+extern FXStringDict* mtdevices;
+extern FXStringDict* updevices;
+extern FXbool mount_messages;
 #endif
 
 
-// Maximum length of a file name
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 1024
-#endif
 
 // Delay before the progress bar should be shown (ms)
 #define SHOW_PBAR_DELAY 1000
@@ -51,15 +55,69 @@ extern FXbool confirm_overwrite;
 // Timeout for stat functions (seconds)
 #define TIMEOUT 5
 
+// Safe strcpy function (Public domain, by C.B. Falconer)
+// The destination string is always null terminated
+size_t strlcpy(char *dst, const char *src, size_t sz)
+{
+	const char *start = src;
+
+	if (src && sz--)
+	{
+		while ((*dst++ = *src))
+			if (sz--)
+				src++;
+			else
+		 	{
+            	*(--dst) = '\0';
+            	break;
+         	}
+   	}
+   	if (src)
+   	{
+      	while (*src++)
+			continue;
+      	return src - start - 1;
+   	}
+   	else if (sz)
+		*dst = '\0';
+	return 0;
+}
+
+
+// Safe strcat function (Public domain, by C.B. Falconer)
+// The destination string is always null terminated
+size_t strlcat(char *dst, const char *src, size_t sz)
+{
+   char  *start = dst;
+
+   while (*dst++)      // assumes sz >= strlen(dst)
+      if (sz)
+	  	sz--;          // i.e. well formed string
+   dst--;
+   return dst - start + strlcpy(dst, src, sz);
+}
+
+
+// To test if two strings are equal (strcmp replacement, thanks to Francesco Abbate)
+int streq(const FXchar *a, const FXchar *b)
+{
+	if (a == NULL || b == NULL)
+		return 0;
+	return (strcmp(a, b) == 0);
+}
+
+
+
 // Replacement of the stat function with a timeout
 int statout(const char *filename, struct stat *buf)
 {
 #if defined(linux)
 	// It's a mount point
-	if(mounts->find(filename))
+	if(mtdevices->find(filename))
 	{
-		// Mount point is NFS or SMB
-		if(!strcmp(mounts->find(filename),"nfs") || !strcmp(mounts->find(filename),"smbfs"))
+		// Mount point is up and is NFS or SMB
+		if(::streq(updevices->find(filename),"up")
+		   && (::streq(mtdevices->find(filename),"nfs") || ::streq(mtdevices->find(filename),"smbfs")))
 		{
 			int pid, ret;
 			time_t t0, t;
@@ -108,7 +166,7 @@ int statout(const char *filename, struct stat *buf)
 			}
 		}
 		// Mount point is down
-		else if(!strcmp(mounts->find(filename),"down"))
+		else if(::streq(updevices->find(filename),"down"))
 			return -1;
 		
 		// Other mount point
@@ -126,11 +184,12 @@ int statout(const char *filename, struct stat *buf)
 int lstatout(const char *filename, struct stat *buf)
 {
 #if defined(linux)
-	// It's a mount point and it's up 
-	if(mounts->find(filename))
+	// It's a mount point
+	if(mtdevices->find(filename))
 	{
 		// Mount point is NFS or SMB
-		if(!strcmp(mounts->find(filename),"nfs") || !strcmp(mounts->find(filename),"smbfs"))
+		if(::streq(updevices->find(filename),"up")
+		   && (::streq(mtdevices->find(filename),"nfs") || ::streq(mtdevices->find(filename),"smbfs")))
 		{
 			int pid, ret;
 			time_t t0, t;
@@ -179,7 +238,7 @@ int lstatout(const char *filename, struct stat *buf)
 			}
 		}
 		// Mount point is down
-		else if(!strcmp(mounts->find(filename),"down"))
+		else if(::streq(updevices->find(filename),"down"))
 			return -1;
 		
 		// Other mount point
@@ -255,7 +314,7 @@ unsigned long long dirsize(char *path)
 	struct stat statbuf;
 	char buf[MAXPATHLEN];
 	unsigned long long dsize=0;
-	int l;
+	int l, ret;
 			
 	if((dp=opendir(path))==NULL)
         return 0;
@@ -266,25 +325,22 @@ unsigned long long dirsize(char *path)
 
 	while((dirp=readdir(dp)))
     {
-        if ( !strcmp(dirp->d_name, ".")||!strcmp(dirp->d_name, "..") )
+        if ( ::streq(dirp->d_name, ".")||::streq(dirp->d_name, "..") )
             continue;
 		
-		sprintf(buf,"%s/%s",path,dirp->d_name);
+		snprintf(buf,sizeof(buf)-1,"%s/%s",path,dirp->d_name);
 		
 #if defined(linux)
-		if (!mounts->find(buf))
+		if (mtdevices->find(buf))
+			ret=lstatout(buf,&statbuf);
+		else
+#endif
+			ret=lstat(buf,&statbuf);
+		if(ret>=0)
 		{
-#endif
- 			if(lstat(buf,&statbuf)<0)
-        		fprintf(stderr,"Dirsize : cannot stat %s\n", buf);
-			else
-			{
-				if(!S_ISDIR(statbuf.st_mode))
-		  			dsize += (unsigned long long) statbuf.st_size;
-			}
-#if defined(linux)
+			if(!S_ISDIR(statbuf.st_mode))
+		  		dsize += (unsigned long long) statbuf.st_size;
 		}
-#endif
 	}
 	if(closedir(dp)<0)
         fprintf(stderr,"Cannot close directory %s\n", path);
@@ -302,16 +358,13 @@ unsigned long long dirpath( char *path)
 	int ret;
 
 #if defined(linux)
-    if(mounts->find(path))
+	if (mtdevices->find(path))
 		ret=lstatout(path,&statbuf);
 	else
 #endif
 		ret=lstat(path,&statbuf);
 	if(ret<0)
-    {
-        fprintf(stderr,"Dirpath : cannot stat %s\n", path);
         return 0;
-    }
     dsize = (unsigned long long) statbuf.st_size;
     
 	// Not a directory
@@ -330,9 +383,9 @@ unsigned long long dirpath( char *path)
     
 	while((dirp=readdir(dp)))
     {
-        if( !strcmp(dirp->d_name, ".")||!strcmp(dirp->d_name, "..") )
+        if( ::streq(dirp->d_name, ".") || ::streq(dirp->d_name, "..") )
             continue;
-        strcpy(ptr, dirp->d_name);
+        strlcpy(ptr, dirp->d_name, strlen(dirp->d_name)+1);
 		
 		// Recursive call
         dsize+=dirpath(path);
@@ -346,42 +399,48 @@ unsigned long long dirpath( char *path)
 }
 
 // Write the file size in human readable form (bytes or Kbytes or Mbytes)
-void hSize(FXchar* size,FXchar *hsize)
+FXString hSize(FXchar* size)
 {
     int flag=0;
-    FXchar suf[50];
+    FXchar suf[64];
+	FXchar buf[128];
+	FXString hsize;
 
     unsigned long long lsize = strtoull(size,NULL,10);
     float fsize=0.0;
 
-	strcpy(suf,_("bytes")); 
+	strlcpy(suf,_("bytes"),sizeof(suf)); 
 	if(lsize>1000000000)
     {
         fsize=lsize/1073741824.0;
-        strcpy(suf,_("GB"));
+        strlcpy(suf,_("GB"),sizeof(suf));
         flag=1;
     }
 	else if(lsize>1000000)
     {
         fsize=lsize/1048576.0;
-        strcpy(suf,_("MB"));
+        strlcpy(suf,_("MB"),sizeof(suf));
         flag=1;
     }
     else if(lsize>1000)
     {
         fsize=lsize/1024.0;
-        strcpy(suf,_("KB"));
+        strlcpy(suf,_("KB"),sizeof(suf));
         flag=1;
     }
-    if(flag)
+  	
+	if(flag)
     {
-        if(fsize==(int)fsize)
-            sprintf(hsize,"%.0f %s",fsize,suf);
+		if(fsize==(int)fsize)
+            snprintf(buf,sizeof(buf),"%.0f %s",fsize,suf);
         else
-            sprintf(hsize,"%.1f %s",fsize,suf);
+            snprintf(buf,sizeof(buf),"%.1f %s",fsize,suf);
     }
     else
-        sprintf(hsize,"%llu %s",lsize,suf);
+        snprintf(buf,sizeof(buf),"%llu %s",lsize,suf);
+	
+	hsize=buf;
+	return hsize;
 }
 
 
@@ -435,6 +494,21 @@ FXString filePath(const FXString path, const FXString dir)
 		return (dir+PATHSEPSTRING+out);
 }
 
+
+// Obtain a unique file name based on the current time stamp, such as : filename__YYYYMMDD-HH:MM:SS-sec
+// If filename is a path name, only the modified filename is returned
+// Used for file deletion to trash can
+FXString stampname(const FXString filename)
+{
+	struct timeval tv;
+	char stamp[32];
+	
+	gettimeofday(&tv,NULL);
+	snprintf(stamp,sizeof(stamp)-1,"__%s-%ld",FXFile::time("%Y%m%d-%H:%M:%S",tv.tv_sec).text(),tv.tv_usec);
+	return (FXFile::name(filename)+stamp);
+}
+
+
 // Enquote filename to make it safe for shell
 // (the function provided by Fox seems to be buggy??)
 FXString quote(const FXString& file,FXbool forcequotes)
@@ -537,10 +611,10 @@ FXint isEmptyDir(const FXString directory)
 FXbool exists(const FXString& file)
 {
   	struct stat status;
-  	return !file.empty() && (statout(file.text(),&status)==0);
+  	return !file.empty() && (lstatout(file.text(),&status)==0);
 }
 
-// Check if file represents a directory
+// Check if the file represents a directory
 FXbool isDirectory(const FXString& file)
 {
   	struct stat status;
@@ -554,6 +628,154 @@ FXbool isFile(const FXString& file)
   	return !file.empty() && (statout(file.text(),&status)==0) && S_ISREG(status.st_mode);
 }
 
+// Check if the file or the link refered file is writable
+FXbool isWritable(const FXString& file)
+{
+  	struct stat status;
+	FXuint uid=getuid();
+	FXuint gid=getgid();
+	
+	// File exists and can be stated
+	if (!file.empty() && (statout(file.text(),&status)==0))
+	{
+		// We are root
+		if (uid==0)
+		{
+			return TRUE;
+		}
+		// We are owner of the file
+		else if (uid==status.st_uid)
+		{
+			// The file is writable
+			if (status.st_mode & S_IWUSR)
+				return TRUE;
+			else
+				return FALSE;
+		}
+		// We are in the group of the file
+		else if (gid==status.st_gid)
+		{
+			// The file is writable
+			if (status.st_mode & S_IWGRP)
+				return TRUE;
+			else
+				return FALSE;
+		}
+		// We are in others
+		else
+		{
+			// The file is writable
+			if (status.st_mode & S_IWOTH)
+				return TRUE;
+			else
+				return FALSE;
+		}
+	}	
+	
+	// File doesn't exist or cannot be stated
+	else
+		return FALSE;
+}
+
+// Check if the file or the link itself is writable
+FXbool isLWritable(const FXString& file)
+{
+  	struct stat status;
+	FXuint uid=getuid();
+	FXuint gid=getgid();
+	
+	// File or link exists and can be stated
+	if (!file.empty() && (lstatout(file.text(),&status)==0))
+	{
+		// We are root
+		if (uid==0)
+		{	
+			return TRUE;
+		}
+		// We are owner of the file or link
+		else if (uid==status.st_uid)
+		{
+			// The file or link is writable
+			if (status.st_mode & S_IWUSR)
+				return TRUE;
+			else
+				return FALSE;
+		}
+		// We are in the group of the file or link
+		else if (gid==status.st_gid)
+		{
+			// The file or link is writable
+			if (status.st_mode & S_IWGRP)
+				return TRUE;
+			else
+				return FALSE;
+		}
+		// We are other
+		else
+		{
+			// The file or link is writable
+			if (status.st_mode & S_IWOTH)
+				return TRUE;
+			else
+				return FALSE;
+		}
+	}	
+	
+	// File or link doesn't exist or cannot be stated
+	else
+		return FALSE;
+}
+
+// Check if the file or the link refered file is readable AND executable
+// Function used to test if we can enter a directory
+FXbool isReadExecutable(const FXString& file)
+{
+  	struct stat status;
+	FXuint uid=getuid();
+	FXuint gid=getgid();
+	
+	// File exists and can be stated
+	if (!file.empty() && (statout(file.text(),&status)==0))
+	{
+		// We are root
+		if (uid==0)
+		{
+			return TRUE;
+		}
+		// We are owner of the file
+		else if (uid==status.st_uid)
+		{
+			// The file is readable and executable
+			if ((status.st_mode & S_IRUSR) && (status.st_mode & S_IXUSR))
+				return TRUE;
+			else
+				return FALSE;
+		}
+		// We are in the group of the file
+		else if (gid==status.st_gid)
+		{
+			// The file is readable and executable
+			if ((status.st_mode & S_IRGRP) && (status.st_mode & S_IXGRP))
+				return TRUE;
+			else
+				return FALSE;
+		}
+		// We are in others
+		else
+		{
+			// The file is readable and executable
+			if ((status.st_mode & S_IROTH) && (status.st_mode & S_IXOTH))
+				return TRUE;
+			else
+				return FALSE;
+		}
+	}	
+	
+	// File doesn't exist or cannot be stated
+	else
+		return FALSE;
+}
+
 // Check if file represents a link
 FXbool isLink(const FXString& file)
 {
@@ -579,6 +801,18 @@ FXString readlink(const FXString& file)
 }
 
 
+// Return true if files are identical
+// Compare file names and inodes for case insensitive filesystems
+FXbool identical(const FXString& file1,const FXString& file2)
+{
+	if(file1!=file2)
+	{
+    	struct stat stat1,stat2;
+    	return !::lstatout(file1.text(),&stat1) && !::lstatout(file2.text(),&stat2) && stat1.st_ino==stat2.st_ino && stat1.st_dev==stat2.st_dev;
+    }
+  	return TRUE;
+}
+
 
 // Message Map
 FXDEFMAP(File) FileMap[]={
@@ -590,7 +824,7 @@ FXDEFMAP(File) FileMap[]={
 FXIMPLEMENT(File,FXDialogBox,FileMap,ARRAYNUMBER(FileMap))
 
 // Construct object
-File::File(FXApp* a, FXString title, const unsigned int operation):FXDialogBox(a,title,DECOR_ALL)
+File::File(FXWindow *owner, FXString title, const unsigned int operation):FXDialogBox(owner,title,DECOR_TITLE|DECOR_BORDER)
 {
     // Progress window
 	FXPacker *buttons=new FXPacker(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X,0,0,10,10,PDIALOG_WIDTH,PDIALOG_WIDTH,5,5);
@@ -604,9 +838,6 @@ File::File(FXApp* a, FXString title, const unsigned int operation):FXDialogBox(a
     cancelButton->addHotKey(KEY_Escape);
 	isCancelled=FALSE;
 
-	// Timer
-	timer=NULL;
-	
 	// Progress bar
 	pbar=NULL;
 
@@ -620,16 +851,17 @@ File::File(FXApp* a, FXString title, const unsigned int operation):FXDialogBox(a
 			pbar=new FXProgressBar(contents,NULL,0,LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|PROGRESSBAR_PERCENTAGE,0,0,0,0,PDIALOG_WIDTH);
 		
 			// Timer on
-			timer=getApp()->addTimeout(this,File::ID_TIMEOUT,SHOW_PBAR_DELAY);
+			getApp()->addTimeout(this,File::ID_TIMEOUT,SHOW_PBAR_DELAY);
 			break;
 
     	case MOVE:
-	    	// Labels
+	    	// Labels and progress bar
 			uplabel=new FXLabel(contents,_("Source :"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_X);
     		downlabel=new FXLabel(contents,_("Target :"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_X);
-		
+			pbar=new FXProgressBar(contents,NULL,0,LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|PROGRESSBAR_PERCENTAGE,0,0,0,0,PDIALOG_WIDTH);
+
 			// Timer on
-			timer=getApp()->addTimeout(this,File::ID_TIMEOUT,SHOW_PBAR_DELAY);
+			getApp()->addTimeout(this,File::ID_TIMEOUT,SHOW_PBAR_DELAY);
 			break;
 
     	case DELETE:
@@ -638,7 +870,7 @@ File::File(FXApp* a, FXString title, const unsigned int operation):FXDialogBox(a
     		downlabel=new FXLabel(contents,_("From :"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_X);
 
 			// Timer on
-			timer=getApp()->addTimeout(this,File::ID_TIMEOUT,SHOW_PBAR_DELAY);
+			getApp()->addTimeout(this,File::ID_TIMEOUT,SHOW_PBAR_DELAY);
 			break;
 
     	case CHMOD:
@@ -647,7 +879,7 @@ File::File(FXApp* a, FXString title, const unsigned int operation):FXDialogBox(a
     		downlabel=new FXLabel(contents,_("File :"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_X);
 
 			// Timer on
-			timer=getApp()->addTimeout(this,File::ID_TIMEOUT,SHOW_PBAR_DELAY);
+			getApp()->addTimeout(this,File::ID_TIMEOUT,SHOW_PBAR_DELAY);
 			break;
 
     	case CHOWN:
@@ -656,7 +888,7 @@ File::File(FXApp* a, FXString title, const unsigned int operation):FXDialogBox(a
     		downlabel=new FXLabel(contents,_("File :"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_X);
 
 			// Timer on
-			timer=getApp()->addTimeout(this,File::ID_TIMEOUT,SHOW_PBAR_DELAY);
+			getApp()->addTimeout(this,File::ID_TIMEOUT,SHOW_PBAR_DELAY);
 			break;
 
 #if defined(linux)
@@ -697,8 +929,7 @@ File::File(FXApp* a, FXString title, const unsigned int operation):FXDialogBox(a
 // Destructor
 File::~File()
 {
-	getApp()->removeTimeout(timer);
-	timer=NULL;
+	getApp()->removeTimeout(this,File::ID_TIMEOUT);
     delete pbar;
 }
 
@@ -710,17 +941,16 @@ void File::create()
 	FXDialogBox::create();
 }
 
-
 // Force check of timeout for progress dialog (to avoid latency problems)
-int File::checkTimeout(FXTimer *timer)
+int File::checkTimeout(void)
 {
 	timeval now;	
-	if (timer)
+	if (getApp()->hasTimeout(this,File::ID_TIMEOUT))
 	{
-		if (getApp()->remainingTimeout(timer)==0)
+		if (getApp()->remainingTimeout(this,File::ID_TIMEOUT)==0)
 			if (now.tv_sec>0)
 			{
-				getApp()->removeTimeout(timer);
+				getApp()->removeTimeout(this,File::ID_TIMEOUT);
 				if (allowPdialog)
 					show(PLACEMENT_OWNER);
 				getApp()->forceRefresh();
@@ -731,7 +961,6 @@ int File::checkTimeout(FXTimer *timer)
 	return 0;
 }
 
-
 // Read bytes
 long File::fullread(int fd,unsigned char *ptr,long len)
 {
@@ -787,7 +1016,7 @@ FXbool File::copyfile(const FXString& source, const FXString& target)
         if(statout(source.text(),&status)==0)
         {
 			// If destination is a directory
-			if(isDirectory(target))
+			if(::isDirectory(target))
 				destfile=target+PATHSEPSTRING+FXFile::name(source);
 			else
 				destfile=target;
@@ -808,8 +1037,7 @@ FXbool File::copyfile(const FXString& source, const FXString& target)
                         break;
 											
 					// Force timeout checking for progress dialog
-					if(checkTimeout(timer))
-						timer=NULL;
+					checkTimeout();
 
 					// Set percentage value for progress dialog
 					nread_all+=nread;
@@ -863,7 +1091,7 @@ FXbool File::copydir(const FXString& source,const FXString& target,struct stat&
     FXString destfile,oldchild,newchild;
 
 	// If destination is an existing directory
-	if(isDirectory(target))
+	if(::isDirectory(target))
 		destfile=target+PATHSEPSTRING+FXFile::name(source);
 	else
 		destfile=target;
@@ -905,10 +1133,33 @@ FXbool File::copydir(const FXString& source,const FXString& target,struct stat&
             if(!ISPATHSEP(newchild[newchild.length()-1]))
                 newchild.append(PATHSEP);
             newchild.append(dp->d_name);
-            if(!copyrec(oldchild,newchild,&inode))
+			if(!copyrec(oldchild,newchild,&inode))
             {
-                closedir(dirp);
-                return FALSE;
+				// If the cancel button was pressed
+				if (isCancelled)
+				{
+                	closedir(dirp);
+                	return FALSE;
+				}
+				
+				// Or a permission problem occured
+				else
+				{
+					FXPNGIcon icon(getApp(),errorIcon);
+					FXString str;
+					if (::isDirectory(oldchild))
+						str=_("Can't copy folder ") + oldchild + _(" : Permission denied");
+					else
+						str=_("Can't copy file ") + oldchild + _(" : Permission denied");
+       				MessageBox box(this,_("Error"),str,&icon,MBOX_OK_CANCEL|DECOR_TITLE|DECOR_BORDER);        	
+					FXuint answer=box.execute(PLACEMENT_SCREEN);
+					if(answer == MBOX_CLICKED_CANCEL)
+					{
+               			closedir(dirp);
+						isCancelled=TRUE;
+                		return FALSE;
+					}					
+				}
             }
         }
     }
@@ -926,7 +1177,7 @@ FXbool File::copyrec(const FXString& source,const FXString& target,inodelist* in
 {
     struct stat status1,status2;
 
-    // Old file or directory does not exist
+    // Source file or directory does not exist
     if(lstatout(source.text(),&status1)!=0)
         return FALSE;
 
@@ -972,33 +1223,42 @@ FXbool File::copyrec(const FXString& source,const FXString& target,inodelist* in
 
 
 // Copy file
-FXbool File::copy(const FXString& source,const FXString& target)
+FXbool File::copy(const FXString& source, const FXString& target, const FXbool confirm_dialog)
 {
 	FXString targetfile;
 
 	// Source doesn't exist
-	if(!exists(dequote(source)))
+	if(!::exists(source))
+	{
+		MessageBox::error(this,MBOX_OK,_("Error"),_("Source %s doesn't exist"), source.text());
 		return FALSE;
+	}
 
 	// Source and target are identical
-	if (target==source)
+	if (::identical(target,source))
+	{
+		MessageBox::error(this,MBOX_OK,_("Error"),_("Source %s is identical to target"), target.text());
 		return FALSE;
+	}
 
 	// Target is an existing directory
-	if (isDirectory(target))
+	if (::isDirectory(target))
 		targetfile=target+PATHSEPSTRING+FXFile::name(source);
 	else
 		targetfile=target;
 
 	// Source and target file are identical
-	if (targetfile==source)
+	if (::identical(targetfile,source))
+	{
+		MessageBox::error(this,MBOX_OK,_("Error"),_("Source %s is identical to target"), targetfile.text());			
 		return FALSE;
+	}
 
 	// Target already exists
-	if(exists(dequote(targetfile)))
+	if(::exists(targetfile))
 	{
 		// Overwrite dialog if necessary
-		if (!(overwrite_all | skip_all))
+		if (!(overwrite_all | skip_all) & confirm_dialog)
     	{
 			FXString label=_("Source : ")+source;
 			uplabel->setText(label);
@@ -1065,8 +1325,7 @@ FXbool File::remove(const FXString& file)
 				ISDIR=TRUE;
 				
 				// Force timeout checking for progress dialog
-				if(checkTimeout(timer))
-					timer=NULL;
+				checkTimeout();
 
 				// Give cancel button an opportunity to be clicked
 				if(cancelButton)
@@ -1115,8 +1374,7 @@ FXbool File::remove(const FXString& file)
 			if (!ISDIR)
 			{
 				// Force timeout checking for progress dialog
-				if(checkTimeout(timer))
-					timer=NULL;
+				checkTimeout();
 
 				// Give cancel button an opportunity to be clicked
 				if(cancelButton)
@@ -1148,18 +1406,24 @@ FXbool File::remove(const FXString& file)
 
 
 // Rename a file or a directory
-FXbool File::rename(const FXString& source,const FXString& target)
+FXbool File::rename(const FXString& source, const FXString& target)
 {
 	// Source doesn't exist
-    if(!exists(dequote(source)))
+    if(!::exists(source))
+	{
+		MessageBox::error(this,MBOX_OK,_("Error"),_("Source %s doesn't exist"), source.text());
 		return FALSE;
+	}
 			
 	// Source and target are identical
-	if (target==source)
+	if (::identical(target,source))
+	{
+		MessageBox::error(this,MBOX_OK,_("Error"),_("Source %s is identical to target"), target.text());
 		return FALSE;
+	}
 
 	// Target already exists
-	if(exists(dequote(target)))
+	if(::exists(target))
 	{
 		// Overwrite dialog if necessary
 		if (!(overwrite_all | skip_all))
@@ -1186,14 +1450,6 @@ FXbool File::rename(const FXString& source,const FXString& target)
 		}
 		if(!(overwrite | overwrite_all) | skip_all)
 			return FALSE;
-
-		// Use of the standard C function to remove the target
-		// avoids to overwrite a non empty directory
-		if (::remove(target.text())==-1)
-		{
-			MessageBox::error(this,MBOX_OK,_("Error"),_("Cannot overwrite the non empty folder %s"), target.text());			
-			return FALSE;
-		}
 	}
        		
 	// Rename file using the standard C function
@@ -1207,67 +1463,65 @@ FXbool File::rename(const FXString& source,const FXString& target)
 	}
 
 	// If files are on different file systems, use the copy/delete scheme
-	// and use a progress bar on top of the progress dialog when file copy is lengthy
-	FXString label=_("Source : ")+source;
-	uplabel->setText(label);
-	label=_("Target : ")+target;
-	downlabel->setText(label);
-	getApp()->repaint();
-	File *f=NULL;
-	f=new File(getApp(),_("File move"),COPY);
-	f->create();
-	FXbool ret=f->copy(source,target);
-
-	// If action is cancelled in progress dialog
-	if (f->isCancelled)
-	{
-		f->hide();
-		MessageBox::error(this,MBOX_OK,_("Error"),_("Copy file operation cancelled!"));
-		delete f;
-		return FALSE;
-	}
-	delete f;
+	FXbool ret=this->copy(source,target,FALSE);
 	if (ret)
 		return (remove(source.text())==TRUE);
 	else
 		return FALSE;
 }
 
+
 // Move files
 FXbool File::move(const FXString& source,const FXString& target)
 {
 	// Source doesn't exist
-	if(!exists(dequote(source)))
+	if(!::exists(source))
+	{
+		MessageBox::error(this,MBOX_OK,_("Error"),_("Source %s doesn't exist"), source.text());
 		return FALSE;
+	}
 
 	// Source and target are identical
-	if (target==source)
+	if (identical(target,source))
+	{
+		MessageBox::error(this,MBOX_OK,_("Error"),_("Source %s is identical to target"), target.text());
 		return FALSE;
+	}
 
 	// Target is an existing directory
 	FXString targetfile;
-	if (isDirectory(target))
+	if (::isDirectory(target))
 		targetfile=target+PATHSEPSTRING+FXFile::name(source);
 	else
 		targetfile=target;
 
 	// Source and target file are identical
-	if (targetfile==source)
+	if (::identical(targetfile,source))
+	{
+		MessageBox::error(this,MBOX_OK,_("Error"),_("Source %s is identical to target"), target.text());
 		return FALSE;
+	}
+
+	// Force timeout checking for progress dialog
+	checkTimeout();
+
+	// Give cancel button an opportunity to be clicked
+	if(cancelButton)
+		getApp()->runModalWhileEvents(cancelButton);
+
+	// Set labels for progress dialog
+	FXString label=_("Source : ")+source;
+	uplabel->setText(label);
+	label=_("Target : ")+target;
+	downlabel->setText(label);
+	getApp()->repaint();
 
 	// Target file already exists
-	if(exists(dequote(targetfile)))
+	if(::exists(targetfile))
     {
 		// Overwrite dialog if necessary
 		if (!overwrite_all & !skip_all)
 		{
-			// Set labels for progress dialog
-			FXString label=_("Source : ")+source;
-			uplabel->setText(label);
-			label=_("Target : ")+target;
-			downlabel->setText(label);
-			getApp()->repaint();
-
 			FXString msg=_("File ")+targetfile+_(" already exists. Overwrite?");
        		ConfirmOverWriteBox* dlg = new ConfirmOverWriteBox(this,_("Confirm Overwrite"),msg);
        		FXuint answer=dlg->execute();
@@ -1291,17 +1545,28 @@ FXbool File::move(const FXString& source,const FXString& target)
 		}
 		if(!(overwrite | overwrite_all) | skip_all)
 			return FALSE;
-			
-		// Use of the standard C function to remove the target
-		// avoids to overwrite a non empty directory
-		if (::remove(targetfile.text())==-1)
-		{
-			MessageBox::error(this,MBOX_OK,_("Error"),_("Cannot overwrite the non empty folder %s"), targetfile.text());			
-			return FALSE;
-		}
 	}
-	// Rename file
-	return File::rename(source,targetfile);
+
+	// Set the progress to 100% for move file operation
+	if (pbar)
+		pbar->setProgress(100);
+
+	// Rename file using the standard C function
+	// This should only work for files that are on the same file system
+	if (::rename(source.text(),targetfile.text())==0)
+		return TRUE;
+	if(errno!=EXDEV)
+	{
+		MessageBox::error(this,MBOX_OK,_("Error"),_("Cannot rename to target %s"), targetfile.text());	
+		return FALSE;
+	}
+
+	// If files are on different file systems, use the copy/delete scheme
+	FXbool ret=this->copy(source,targetfile,FALSE);
+	if (ret)
+		return (remove(source.text())==TRUE);
+	else
+		return FALSE;
 }
 
 
@@ -1309,15 +1574,21 @@ FXbool File::move(const FXString& source,const FXString& target)
 FXbool File::symlink(const FXString& source,const FXString& target)
 {
 	// Source doesn't exist
-    if(!exists(dequote(source)))
+    if(!::exists(source))
+	{
+		MessageBox::error(this,MBOX_OK,_("Error"),_("Source %s doesn't exist"), source.text());
 		return FALSE;
+	}
 			
 	// Source and target are identical
-	if (target==source)
+	if (::identical(target,source))
+	{
+		MessageBox::error(this,MBOX_OK,_("Error"),_("Source %s is identical to target"), target.text());
 		return FALSE;
+	}
 
 	// Target already exists
-	if(exists(dequote(target)))
+	if(::exists(target))
 	{
 		// Overwrite dialog if necessary
 		if (!(overwrite_all | skip_all))
@@ -1344,14 +1615,6 @@ FXbool File::symlink(const FXString& source,const FXString& target)
 		}
 		if(!(overwrite | overwrite_all) | skip_all)
 			return FALSE;
-
-		// Use of the standard C function to remove the target
-		// Avoids to overwrite a non empty directory
-		if (::remove(target.text())==-1)
-		{
-			MessageBox::error(this,MBOX_OK,_("Error"),_("Cannot overwrite the non empty folder %s"), target.text());			
-			return FALSE;
-		}
 	}
     
 	// Create symbolic link using the standard C function
@@ -1370,12 +1633,12 @@ FXbool File::symlink(const FXString& source,const FXString& target)
 // Note : the variable file returns the last processed file
 // It can be different from the initial path, if recursive chmod is used
 // (Used to fill an error message, if needed) 
-int File::chmod(char *path, char *file, mode_t mode, FXbool rec, FXbool dironly)
+int File::chmod(char *path, char *file, mode_t mode, FXbool rec, const FXbool dironly)
 {		
     struct stat stats;
 
 	// Initialise the file variable with the initial path
-	strcpy(file, path);
+	strlcpy(file, path, strlen(path)+1);
 
     if (lstatout(path, &stats)) //if it doesn't exist
         return -1;
@@ -1386,8 +1649,7 @@ int File::chmod(char *path, char *file, mode_t mode, FXbool rec, FXbool dironly)
             return 0;
 			
 		// Force timeout checking for progress dialog
-		if(checkTimeout(timer))
-			timer=NULL;
+		checkTimeout();
 
 		// Give cancel button an opportunity to be clicked
 		if(cancelButton)
@@ -1411,8 +1673,7 @@ int File::chmod(char *path, char *file, mode_t mode, FXbool rec, FXbool dironly)
         if(rec == FALSE)
         {
 			// Force timeout checking for progress dialog
-			if(checkTimeout(timer))
-				timer=NULL;
+			checkTimeout();
 
 			// Give cancel button an opportunity to be clicked
 			if(cancelButton)
@@ -1441,12 +1702,12 @@ int File::chmod(char *path, char *file, mode_t mode, FXbool rec, FXbool dironly)
 
 
 // Recursive chmod for a directory
-int File::rchmod(char *path, char *file, mode_t mode, FXbool dironly)
+int File::rchmod(char *path, char *file, mode_t mode, const FXbool dironly)
 {
     struct stat stats;
 
 	// Initialise the file variable with the initial path
-	strcpy(file, path);
+	strlcpy(file, path, strlen(path)+1);
 
     if (lstatout(path, &stats)) //if it doesn't exist
         return -1;
@@ -1457,8 +1718,7 @@ int File::rchmod(char *path, char *file, mode_t mode, FXbool dironly)
             return 0;
 
 		// Force timeout checking for progress dialog
-		if(checkTimeout(timer))
-			timer=NULL;
+		checkTimeout();
 
 		// Give cancel button an opportunity to be clicked
 		if(cancelButton)
@@ -1494,13 +1754,13 @@ int File::rchmod(char *path, char *file, mode_t mode, FXbool dironly)
             int pl1 = pl, l = strlen(entry->d_name);
             char *path1 = (char *)alloca(pl1+l+2);
 
-            strcpy(path1, path);
+            strlcpy(path1, path, strlen(path)+1);
             if (path1[pl1-1] != '/')
                 path1[pl1++] = '/';
-            strcpy(path1+pl1, entry->d_name);
+            strlcpy(path1+pl1, entry->d_name, strlen(entry->d_name)+1);
 
 			// Modify the file variable with the new path
-			strcpy(file, path1);
+			strlcpy(file, path1, strlen(path1)+1);
             if (rchmod(path1,file,mode,dironly))
                 return -1;
         }
@@ -1517,12 +1777,12 @@ int File::rchmod(char *path, char *file, mode_t mode, FXbool dironly)
 // Note : the variable file returns the last processed file
 // It can be different from the initial path, if recursive chmod is used
 // (Used to fill an error message, if needed) 
-int File::chown(char *path, char *file, uid_t uid, gid_t gid, FXbool rec, FXbool dironly)
+int File::chown(char *path, char *file, uid_t uid, gid_t gid, const FXbool rec, const FXbool dironly)
 {
     struct stat stats;
 
 	// Initialise the file variable with the initial path
-	strcpy(file, path);
+	strlcpy(file, path, strlen(path)+1);
 
     if (lstatout(path, &stats)) //if it doesn't exist
         return -1;
@@ -1533,8 +1793,7 @@ int File::chown(char *path, char *file, uid_t uid, gid_t gid, FXbool rec, FXbool
             return 0;
 
 		// Force timeout checking for progress dialog
-		if(checkTimeout(timer))
-			timer=NULL;
+		checkTimeout();
 
 		// Give cancel button an opportunity to be clicked
 		if(cancelButton)
@@ -1559,8 +1818,7 @@ int File::chown(char *path, char *file, uid_t uid, gid_t gid, FXbool rec, FXbool
         if(rec == FALSE)
         {
 			// Force timeout checking for progress dialog
-			if(checkTimeout(timer))
-				timer=NULL;
+			checkTimeout();
 
 			// Give cancel button an opportunity to be clicked
 			if(cancelButton)
@@ -1590,12 +1848,12 @@ int File::chown(char *path, char *file, uid_t uid, gid_t gid, FXbool rec, FXbool
 
 
 // Recursive chown for a directory
-int File::rchown(char *path, char *file, uid_t uid, gid_t gid, FXbool dironly)
+int File::rchown(char *path, char *file, uid_t uid, gid_t gid, const FXbool dironly)
 {
     struct stat stats;
 
 	// Initialise the file variable with the initial path
-	strcpy(file, path);
+	strlcpy(file, path, strlen(path)+1);
 
     if (lstatout(path, &stats)) //if it doesn't exist
         return -1;
@@ -1606,8 +1864,7 @@ int File::rchown(char *path, char *file, uid_t uid, gid_t gid, FXbool dironly)
 			return 0;
 
 		// Force timeout checking for progress dialog
-		if(checkTimeout(timer))
-			timer=NULL;
+		checkTimeout();
 
 		// Give cancel button an opportunity to be clicked
 		if(cancelButton)
@@ -1642,11 +1899,11 @@ int File::rchown(char *path, char *file, uid_t uid, gid_t gid, FXbool dironly)
             int pl1 = pl, l = strlen(entry->d_name);
             char *path1 = (char *)alloca(pl1+l+2);
 
-            strcpy(path1, path);
+            strlcpy(path1, path, strlen(path)+1);
             if (path1[pl1-1] != '/')
                 path1[pl1++] = '/';
-            strcpy(path1+pl1, entry->d_name);
-			strcpy(file, path1);
+            strlcpy(path1+pl1, entry->d_name, strlen(entry->d_name)+1);
+			strlcpy(file, path1, strlen(path1)+1);
             if (rchown(path1,file,uid,gid,dironly))
                 return -1;
         }
@@ -1662,7 +1919,7 @@ int File::rchown(char *path, char *file, uid_t uid, gid_t gid, FXbool dironly)
 
 
 // Extract an archive in a specified directory
-int File::extract(FXString name, FXString dir,FXString cmd)
+int File::extract(const FXString name, const FXString dir, const FXString cmd)
 {
 	// Change to the specified directory
     chdir(dir.text());
@@ -1678,10 +1935,10 @@ int File::extract(FXString name, FXString dir,FXString cmd)
 
 
 // Create an archive
-int File::archive(FXString name, FXString cmd)
+int File::archive(const FXString name, const FXString cmd)
 {
 	// Target file already exists
-	if(exists(dequote(name)))
+	if(::exists(name))
     {
 		// Overwrite dialog if necessary
 		if (!overwrite_all & !skip_all)
@@ -1721,7 +1978,7 @@ int File::archive(FXString name, FXString cmd)
 
 
 #if defined(linux)
-int File::mount(FXString dir, FXString msg, FXString cmd, unsigned int op)
+int File::mount(const FXString dir, const FXString msg, const FXString cmd, const unsigned int op)
 {
 	// Show progress dialog (no timer here)
 	show(PLACEMENT_OWNER);
@@ -1754,7 +2011,7 @@ int File::mount(FXString dir, FXString msg, FXString cmd, unsigned int op)
 	FXString buf;
 	while(fgets(text,sizeof(text),pcmd))
 		buf+=text;
-	sprintf(text,"%s",buf.text());
+	snprintf(text,sizeof(text)-1,"%s",buf.text());
 	
 	// Close the stream
 	if(pclose(pcmd))
@@ -1765,18 +2022,20 @@ int File::mount(FXString dir, FXString msg, FXString cmd, unsigned int op)
 	// Hide progress dialog
 	hide();
 	
-	// Success message
-	if (op==MOUNT)
-		MessageBox::information(this,MBOX_OK,_("Success"),_("Folder %s was successfully mounted."),dir.text());
-	else
-		MessageBox::information(this,MBOX_OK,_("Success"),_("Folder %s was successfully unmounted."),dir.text());
-	
+	// Success message, eventually
+	if(mount_messages)
+	{
+		if (op==MOUNT)
+			MessageBox::information(this,MBOX_OK,_("Success"),_("Folder %s was successfully mounted."),dir.text());
+		else
+			MessageBox::information(this,MBOX_OK,_("Success"),_("Folder %s was successfully unmounted."),dir.text());
+	}
 	return 1;
 }
 
 
 // Install / Upgrade RPM package
-int File::rpmInstall(FXString name, FXString cmd)
+int File::rpmInstall(const FXString name, const FXString cmd)
 {
     // Make and show command window
 	CommandWindow *cmdwin=new CommandWindow(getApp(),_("Install/Upgrade RPM"),cmd,10,80);			
@@ -1788,7 +2047,7 @@ int File::rpmInstall(FXString name, FXString cmd)
 }
 
 // Uninstall RPM package
-int File::rpmUninstall(FXString name, FXString cmd)
+int File::rpmUninstall(const FXString name, const FXString cmd)
 {
     // Make and show command window
 	CommandWindow *cmdwin=new CommandWindow(getApp(),_("Uninstall RPM"),cmd,10,80);			
