@@ -10,8 +10,11 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
-#include <fox-1.2/fx.h>
+#include <fox-1.4/fx.h>
 
+size_t strlcpy(char *dst, const char *src, size_t sz);
+size_t strlcat(char *dst, const char *src, size_t sz);
+int streq(const FXchar *a, const FXchar *b);
 int statout(const char *filename, struct stat *buf);
 int lstatout(const char *filename, struct stat *buf);
 #ifdef linux
@@ -19,18 +22,23 @@ int mtlstat(const char *filename, struct stat *buf);
 #endif
 unsigned long long dirsize(char *path);
 unsigned long long dirpath( char *path);
-void hSize(FXchar* size,FXchar *hsize);
+FXString hSize(FXchar* size);
 FXString filePath(const FXString path);
 FXString filePath(const FXString path, const FXString dir);
+FXString stampname(const FXString filename);
 FXint isEmptyDir(const FXString directory);
 FXString quote(const FXString& file,FXbool forcequotes);
 FXString dequote(const FXString& file);
 FXbool exists(const FXString& file);
 FXbool isDirectory(const FXString& file);
 FXbool isFile(const FXString& file);
+FXbool isWritable(const FXString& file);
+FXbool isLWritable(const FXString& file);
+FXbool isReadExecutable(const FXString& file);
 FXbool isLink(const FXString& file);
 FXbool info(const FXString& file,struct stat& inf);
 FXString readlink(const FXString& file);
+FXbool identical(const FXString& file1,const FXString& file2);
 
 // File operations
 enum
@@ -38,7 +46,7 @@ enum
 	COPY,
 	RENAME,
 	MOVE,
-	SLINK,
+	SYMLINK,
 	DELETE,
 	CHMOD,
 	CHOWN,
@@ -63,19 +71,18 @@ class File : public FXDialogBox
 {
     FXDECLARE(File)
 protected:
-	int checkTimeout(FXTimer *timer);
+	int checkTimeout(void);
     long fullread(int fd, unsigned char *ptr, long len);
     long fullwrite(int fd, const unsigned char *ptr, long len);
     FXbool copyfile(const FXString& source, const FXString& target);
 	FXbool copyrec(const FXString& source,const FXString& target,inodelist* inodes);
     FXbool copydir(const FXString& source,const FXString& target,struct stat& parentstatus,inodelist* inodes);
-    int rchmod(char *path, char *file, mode_t mode, FXbool dironly);
-    int rchown(char *path, char *file, uid_t uid, gid_t gid, FXbool dironly);
+    int rchmod(char *path, char *file, mode_t mode, const FXbool dironly);
+    int rchown(char *path, char *file, uid_t uid, gid_t gid, const FXbool dironly);
     FXLabel *uplabel;
     FXLabel *downlabel;
     FXProgressBar *pbar;
 	FXButton *cancelButton;
-	FXTimer *timer;
 	FXbool overwrite;
 	FXbool overwrite_all;
 	FXbool skip_all;
@@ -85,7 +92,7 @@ public:
     {}
 	~File();
     void create();
-    File(FXApp* a, FXString title, const unsigned int operation);
+    File(FXWindow *owner, FXString title, const unsigned int operation);
 
 	enum 
 	{
@@ -96,21 +103,21 @@ public:
 
 	FXbool allowPdialog;
 	FXbool isCancelled;
-    FXbool copy(const FXString& source,const FXString& target);
-    FXbool rename(const FXString& source,const FXString& target);
-    FXbool move(const FXString& source,const FXString& target);
-    FXbool symlink(const FXString& source,const FXString& target);
+    FXbool copy(const FXString& source, const FXString& target, const FXbool confirm_dialog=TRUE);
+    FXbool rename(const FXString& source, const FXString& target);
+    FXbool move(const FXString& source, const FXString& target);
+    FXbool symlink(const FXString& source, const FXString& target);
     FXbool remove(const FXString& file);
 
-    int chmod(char *path, char *file, mode_t mode, FXbool rec, FXbool dironly=FALSE);
-    int chown(char *path, char *file, uid_t uid, gid_t gid, FXbool rec, FXbool dironly=FALSE);
-	int extract(FXString name, FXString dir,FXString cmd);
-	int archive(FXString name, FXString cmd);
+    int chmod(char *path, char *file, mode_t mode, const FXbool rec, const FXbool dironly=FALSE);
+    int chown(char *path, char *file, uid_t uid, gid_t gid, const FXbool rec, const FXbool dironly=FALSE);
+	int extract(const FXString name, const FXString dir,const FXString cmd);
+	int archive(const FXString name, const FXString cmd);
 
 #if defined(linux)
-	int mount(FXString dir, FXString msg, FXString cmd, unsigned int op);
-	int rpmInstall(FXString name, FXString cmd);
-	int rpmUninstall(FXString name, FXString cmd);
+	int mount(const FXString dir, const FXString msg, const FXString cmd,const  unsigned int op);
+	int rpmInstall(const FXString name, const FXString cmd);
+	int rpmUninstall(const FXString name, const FXString cmd);
 #endif
 	long onCmdCancel(FXObject*, FXSelector,void*);
     long onTimeout(FXObject*,FXSelector,void*);
