@@ -20,14 +20,16 @@ protected:
     FXCheckButton *readonly;        // Open file as read only
     FXButton      *accept;          // Accept button
     FXButton      *cancel;          // Cancel button
-    FXIcon        *updiricon;       // Up directory icon
-    FXIcon        *listicon;        // List mode icon
-    FXIcon        *detailicon;      // Detail mode icon
-    FXIcon        *iconsicon;       // Icon mode icon
-    FXIcon        *homeicon;        // Go home icon
-    FXIcon        *workicon;        // Go home icon
-    FXIcon        *shownicon;       // Files shown icon
-    FXIcon        *hiddenicon;      // Files hidden icon
+    FXIcon     *updiricon;       // Up directory icon
+    FXIcon     *listicon;        // List mode icon
+    FXIcon     *detailicon;      // Detail mode icon
+    FXIcon     *iconsicon;       // Icon mode icon
+    FXIcon     *homeicon;        // Go home icon
+    FXIcon     *workicon;        // Go home icon
+    FXIcon     *shownicon;       // Files shown icon
+    FXIcon     *hiddenicon;      // Files hidden icon
+    FXIcon     *showthumbicon;   // Files shown icon
+    FXIcon     *hidethumbicon;   // Files hidden icon
     FXuint         selectmode;      // Select mode
 protected:
     FileSelector()
