@@ -3,10 +3,8 @@
 #include "config.h"
 #include "i18n.h"
 
-#include <fox-1.2/fx.h>
-#include <fox-1.2/FXPNGIcon.h>
-#include <fox-1.2/FXJPGIcon.h>
-#include <fox-1.2/FXTIFIcon.h>
+#include <fox-1.4/fx.h>
+#include <fox-1.4/FXPNGIcon.h>
 
 #include "icons.h"
 #include "FileDict.h"
@@ -23,12 +21,6 @@
 #define DEFAULTICONPATH   "~/.foxicons:/usr/local/share/icons:/usr/share/icons"
 #endif
 
-// For thumbnail image preview
-#define MAXBIGICONSIZE 64
-#define MAXMINIICONSIZE 20
-
-extern FXbool display_thumbnails;
-
 
 // Object implementation
 FXIMPLEMENT(IconDict,FXDict,NULL,0)
@@ -74,7 +66,6 @@ void *IconDict::createData(const void* ptr)
         // Got icon
         if(icon)
         {
-
             // Find icon in the icon directory
             FXString iconfile=FXFile::search(path,(const char*)ptr);
             if(!iconfile.empty())
@@ -360,77 +351,6 @@ FileAssoc* FileDict::findFileBinding(const FXchar* pathname)
     {
 		if (strlen(filename)>1)
 			record=associate(filename+1);
-
-     	// Thumbnail image preview
-		if(record && display_thumbnails)
-		{
-       		FXIcon* bigicon=NULL;
-       		FXIcon* miniicon=NULL;
-			if(record->extension=="GIF Image")
-			{
-				bigicon=new FXGIFIcon(getApp());
-				miniicon=new FXGIFIcon(getApp());
-			}
-			else if (record->extension=="BMP Image")
-			{
-				bigicon=new FXBMPIcon(getApp());
-				miniicon=new FXBMPIcon(getApp());
-			}
-			else if (record->extension=="X Pixmap")
-			{
-				bigicon=new FXXPMIcon(getApp());
-				miniicon=new FXXPMIcon(getApp());
-			}
-			else if (record->extension=="TIFF Image")
-			{
-				bigicon=new FXTIFIcon(getApp());
-				miniicon=new FXTIFIcon(getApp());
-			}
-			else if (record->extension=="PNG Image")
-			{
-				bigicon=new FXPNGIcon(getApp());
-				miniicon=new FXPNGIcon(getApp());
-			}
-			else if (record->extension=="JPEG Image")
-			{
-				bigicon=new FXJPGIcon(getApp());
-				miniicon=new FXJPGIcon(getApp());
-			}
-			if (bigicon && miniicon)
-			{
-        		FXFileStream str;
-         		if(str.open(pathname,FXStreamLoad))
-				{
-           			bigicon->loadPixels(str);
-           			str.close();
-           			
-					// Eventually scale the big icon
-					if((bigicon->getWidth()>MAXBIGICONSIZE) || (bigicon->getHeight()>MAXBIGICONSIZE))
-					{
-             			if(bigicon->getWidth()>bigicon->getHeight())
-               				bigicon->scale(MAXBIGICONSIZE,(MAXBIGICONSIZE*bigicon->getHeight())/bigicon->getWidth());
-              			else
-               				bigicon->scale((MAXBIGICONSIZE*bigicon->getWidth())/bigicon->getHeight(),MAXBIGICONSIZE);
-             		}
-           			record->bigicon=bigicon;
-           		}
-         		if(str.open(pathname,FXStreamLoad))
-				{
-           			miniicon->loadPixels(str);
-           			str.close();
-           			
-					// Eventually scale the mini icon
-           			if((miniicon->getWidth()>MAXMINIICONSIZE) || (miniicon->getHeight()>MAXMINIICONSIZE))
-					{
-             			if(miniicon->getWidth()>miniicon->getHeight())
-               				miniicon->scale(MAXMINIICONSIZE,(MAXMINIICONSIZE*miniicon->getHeight())/miniicon->getWidth());
-              			else
-               				miniicon->scale((MAXMINIICONSIZE*miniicon->getWidth())/miniicon->getHeight(),MAXMINIICONSIZE);
-             		}
-           			record->miniicon=miniicon;
-           		}
-			}
-		}
 		if(record)
 			return record;
 		filename=strchr(filename+1,'.');
