@@ -13,10 +13,11 @@ class FXAPI FileItem : public FXIconItem
     friend class FileList;
 protected:
     FileAssoc  *assoc;                      /// File association record
+	FileItem   *link;                   	// Link to next item
     unsigned long long size;                /// File size
-    FXTime        date;                       /// File time
+    FXTime        date;                     /// File time
 protected:
-    FileItem():assoc(NULL),size(0),date(0)
+    FileItem():assoc(NULL),link(NULL),size(0),date(0)
     {}
 protected:
     enum{
@@ -30,7 +31,7 @@ protected:
     };
 public:
     // Constructor
-    FileItem(const FXString& text,FXIcon* bi=NULL,FXIcon* mi=NULL,void* ptr=NULL):FXIconItem(text,bi,mi,ptr),assoc(NULL),size(0),date(0)
+    FileItem(const FXString& text,FXIcon* bi=NULL,FXIcon* mi=NULL,void* ptr=NULL):FXIconItem(text,bi,mi,ptr),assoc(NULL),link(NULL),size(0),date(0)
     {}
 
     // Return true if this is a file item
@@ -106,6 +107,7 @@ class FXAPI FileList : public FXIconList
 {
     FXDECLARE(FileList)
 protected:
+	FileItem	*list;                  // File item list
 	FXint 		 prevIndex;
 	FXString     directory;             // Current directory
     FXString     orgdirectory;          // Original directory
@@ -117,15 +119,15 @@ protected:
     FXuint       matchmode;             // File wildcard match mode
     FXTime       timestamp;             // Time when last refreshed
 	FXuint       counter;               // Refresh counter
-    FXTimer     *refreshtimer;          // Refresh timer
-    FXTimer     *opentimer;             // Open up folder when hovering
 	FXbool		ignorecase;				// Case sensitivity for file name sorting
+	FXbool		allowrefresh;			// Allow or disallow periodic refresh
+	FXbool		displaythumbnails;		// Display thumbnails
     FXIcon      *big_folder;            // Big folder icon
     FXIcon      *mini_folder;           // Mini folder icon
     FXIcon      *big_doc;               // Big document icon
     FXIcon      *mini_doc;              // Mini document icon
-    FXIcon      *big_app;               // Big application icon
-    FXIcon      *mini_app;              // Mini application icon
+    FXIcon      *big_exec;              // Big executable icon
+    FXIcon      *mini_exec;             // Mini executable icon
 	FXIcon  	*mini_folderopen;
 	FXIcon  	*mini_folderup;
 	FXIcon  	*mini_lockfolder;
@@ -152,9 +154,10 @@ protected:
     FXIcon  	*mini_cdrom;
     FXIcon  	*big_zip;
     FXIcon  	*mini_zip;
+	FXColor 	*tmpdata;				// Temporary image data
+
 protected:
-    FileList()
-	{}
+    FileList();
     virtual FXIconItem *createItem(const FXString& text,FXIcon *big,FXIcon* mini,void* ptr);
     void listItems();
 private:
@@ -205,6 +208,8 @@ public:
     long onUpdHideHidden(FXObject*,FXSelector,void*);
 	long onCmdHeader(FXObject*,FXSelector,void*);
 	long onUpdHeader(FXObject*,FXSelector,void*);
+	long onCmdToggleThumbnails(FXObject*,FXSelector,void*);
+	long onUpdToggleThumbnails(FXObject* sender,FXSelector,void*);
 public:
 	static FXint ascending(const FXIconItem* pa,const FXIconItem* pb);
 	static FXint descending(const FXIconItem* pa,const FXIconItem* pb);
@@ -236,9 +241,10 @@ public:
         ID_SHOW_HIDDEN,
         ID_HIDE_HIDDEN,
         ID_TOGGLE_HIDDEN,
-        ID_REFRESHTIMER,
+		ID_TOGGLE_THUMBNAILS,
+        ID_REFRESH_TIMER,
 		ID_REFRESH,
-        ID_OPENTIMER,
+        ID_OPEN_TIMER,
         ID_LAST
     };
 public:
@@ -286,10 +292,10 @@ public:
     FXbool isItemExecutable(FXint index) const;
 
 	// Return number of selected items
-	FXint FileList::getNumSelectedItems(void) const;
+	FXint getNumSelectedItems(void) const;
 
 	// Return number of selected items and index of first selected item  
-	FXint FileList::getNumSelectedItems(FXint *index) const;
+	FXint getNumSelectedItems(FXint *index) const;
 	
 	// Return name of item at index
     FXString getItemFilename(FXint index) const;
@@ -310,17 +316,20 @@ public:
     }
 
 	// Return case sensitivity for file name sorting
-	FXbool FileList::getIgnoreCase() const
+	FXbool getIgnoreCase() const
 	{
 		return ignorecase;
 	}
 
 	// Set case sensitivity for file name sorting
-	void FileList::setIgnoreCase(const FXbool icase)
+	void setIgnoreCase(const FXbool icase)
 	{
 		ignorecase=icase;
 	}
 
+	// Allow or disallow periodic refresh
+	void setAllowRefresh(const FXbool allow);
+
     // Change wildcard matching mode
     void setMatchMode(FXuint mode);
 
@@ -330,6 +339,12 @@ public:
     // Show or hide hidden files
     void showHiddenFiles(FXbool showing);
 
+    // Return TRUE if displaying thumbnails
+    FXbool showThumbnails() const;
+
+    // Display or not thumbnails
+    void showThumbnails(FXbool display);
+
     // Return TRUE if showing directories only
     FXbool showOnlyDirectories() const;
 
