@@ -25,6 +25,31 @@ protected:
     FXString 		name;
 	FXbool			ctrlkey;
 	FXbool			selmult;
+	FXString		homelocation;
+	FXString		trashlocation;
+
+    FXIcon			*archicon;
+    FXIcon			*attricon;
+	FXIcon			*newfoldericon;
+	FXIcon			*minidocicon;
+	FXIcon			*delicon;	
+	FXIcon			*copyicon;	
+	FXIcon			*cuticon;	
+	FXIcon			*pasteicon;
+	FXIcon			*symlinkicon;
+	FXIcon			*renameicon;
+	FXIcon			*moveicon;
+	FXIcon			*mapicon;	
+	FXIcon			*unmapicon;	
+	FXIcon			*minixtermicon;
+	FXIcon			*minirpmicon;
+	FXIcon			*locationicon;
+	FXIcon			*entericon;
+	FXIcon			*filtericon;
+	FXIcon			*viewicon;
+	FXIcon			*editicon;
+	FXIcon			*openicon;
+
 
 public:
     FilePanel(FXchar*,FXComposite*, FXuint name_size=200, FXuint size_size=60, FXuint type_size=100, 
@@ -45,10 +70,16 @@ public:
         ID_EDIT,
         ID_PROPERTIES,
         ID_FILE_COPY,
+        ID_FILE_COPYTO,
         ID_FILE_MOVE,
         ID_FILE_RENAME,
-        ID_FILE_SLINK,
+        ID_FILE_SYMLINK,
         ID_FILE_DEL,
+        ID_FILE_ASSOC,
+		ID_MENU,
+		ID_XTERM,
+        ID_EXTRACT,
+        ID_EXTRACT_HERE,
         ID_NEW_DIR,
         ID_NEW_FILE,
         ID_ADD_TO_ARCH,
@@ -62,10 +93,20 @@ public:
         ID_STATUS,
         ID_LABEL,
         ID_REFRESH,
+        ID_SELECT_ALL,
+        ID_DESELECT_ALL,
+        ID_SELECT_INVERSE,
+        ID_SHOW_BIG_ICONS,
+        ID_SHOW_MINI_ICONS,
+        ID_SHOW_DETAILS,
+        ID_TOGGLE_HIDDEN,
+        ID_TOGGLE_THUMBNAILS,
 #if defined(linux)
         ID_MOUNT,
         ID_UMOUNT,
 		ID_RPM_QUERY,
+		ID_RPM_INSTALL,
+		ID_RPM_UNINSTALL,
 #endif
         ID_LAST,
     };
@@ -84,21 +125,37 @@ public:
     long onCmdProperties(FXObject*,FXSelector,void*);
     long onCmdFileMan(FXObject*,FXSelector,void*);
     long onCmdFileDel(FXObject*,FXSelector,void*);
+    long onCmdFileAssoc(FXObject*,FXSelector,void*);
     long onCmdNewDir(FXObject*,FXSelector,void*);
     long onCmdNewFile(FXObject*,FXSelector,void*);
 	long onCmdOpen(FXObject*,FXSelector,void*);
     long onCmdOpenWith(FXObject*,FXSelector,void*);
+	long onCmdXTerm(FXObject*,FXSelector,void*);
+    long onCmdExtract(FXObject*,FXSelector,void*);
+    long onCmdExtractHere(FXObject*,FXSelector,void*);
     long onRefresh(FXObject*,FXSelector,void*);
+    long onCmdSelect(FXObject*,FXSelector,void*);
     long onCmdPopupMenu(FXObject*,FXSelector,void*);
+    long onCmdShow(FXObject*,FXSelector,void*);
+    long onUpdShow(FXObject*,FXSelector,void*);
     long onUpdUp(FXObject*,FXSelector,void*);
     long onUpdPaste(FXObject*,FXSelector,void*);
+    long onCmdToggleHidden(FXObject*,FXSelector,void*);
+    long onUpdToggleHidden(FXObject*,FXSelector,void*);
+	long onCmdToggleThumbnails(FXObject*,FXSelector,void*);
+	long onUpdToggleThumbnails(FXObject*,FXSelector,void*);
     long onCmdAddToArch(FXObject*,FXSelector,void*);
 	long onUpdMenu(FXObject*,FXSelector,void*);
+	long onUpdOpen(FXObject*,FXSelector,void*);
+	long onUpdAddToArch(FXObject*,FXSelector,void*);
 	long onUpdSelMult(FXObject*,FXSelector,void*);
 #if defined(linux)
     long onCmdMount(FXObject*,FXSelector,void*);
     long onUpdMount(FXObject*,FXSelector,void*);
+    long onUpdUnmount(FXObject*,FXSelector,void*);
     long onCmdRpmQuery(FXObject*,FXSelector,void*);
+    long onCmdRpmInstall(FXObject*,FXSelector,void*);
+    long onCmdRpmUninstall(FXObject*,FXSelector,void*);
 #endif
 };
 #endif
