@@ -3,18 +3,15 @@
 #include "config.h"
 #include "i18n.h"
 
-#include <fox-1.2/fx.h>
-#include <fox-1.2/fxkeys.h>
+#include <fox-1.4/fx.h>
+#include <fox-1.4/fxkeys.h>
 
+#include "xfedefs.h"
 #include "FileDialog.h"
 #include "Preferences.h"
 #include "XFileExplorer.h"
 #include "MessageBox.h"
 
-// Root directory string
-#ifndef ROOTDIR
-#define ROOTDIR "/"
-#endif
 
 
 FXbool Theme::operator != (const Theme& t)
@@ -34,10 +31,11 @@ FXbool confirm_del;
 FXbool confirm_overwrite;
 FXbool ask_before_copy;
 FXbool warn_if_root;
+FXbool use_trashcan;
 #if defined(linux)
 FXbool mount_warn;
+FXbool mount_messages;
 #endif
-FXbool display_thumbnails;
 
 // Map
 FXDEFMAP(PreferencesBox) PreferencesMap[]={
@@ -46,7 +44,6 @@ FXDEFMAP(PreferencesBox) PreferencesMap[]={
                                          FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_BROWSE_EDIT,PreferencesBox::onCmdBrowse),
                                          FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_BROWSE_VIEW,PreferencesBox::onCmdBrowse),
                                          FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_BROWSE_XTERM,PreferencesBox::onCmdBrowse),
-                                         FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_BROWSE_CFM,PreferencesBox::onCmdBrowse),
                                          FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_COLOR,PreferencesBox::onCmdColor),
                                          FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_NORMALFONT,PreferencesBox::onCmdNormalFont),
                                          FXMAPFUNC(SEL_COMMAND,PreferencesBox::ID_TEXTFONT,PreferencesBox::onCmdTextFont),
@@ -108,60 +105,54 @@ PreferencesBox::PreferencesBox(FXWindow *win): FXDialogBox(win,_("Preferences"),
 	oldiconpath=getApp()->reg().readStringEntry("SETTINGS","iconpath","/usr/lib/foxicons");
     iconpath->setText(oldiconpath);
 
-    new FXLabel(matrix,_("Default text editor:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    editor = new FXTextField(matrix,15,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_EDIT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//,0,0,0,0,20,20);
-    oldeditor=getApp()->reg().readStringEntry("PROGS","editor","nedit ");
-    editor->setText(oldeditor);
-
-    new FXLabel(matrix,_("Default text viewer:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    new FXLabel(matrix,_("Text viewer for non associated files:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
     viewer = new FXTextField(matrix,15,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
 	new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_VIEW,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//0,0,0,0,20,20);
     oldviewer=getApp()->reg().readStringEntry("PROGS","viewer","xfv ");
     viewer->setText(oldviewer);
 
+    new FXLabel(matrix,_("Text editor for non associated files:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    editor = new FXTextField(matrix,15,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
+    new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_EDIT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//,0,0,0,0,20,20);
+    oldeditor=getApp()->reg().readStringEntry("PROGS","editor","nedit ");
+    editor->setText(oldeditor);
+
     new FXLabel(matrix,_("Terminal program:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
     xterm = new FXTextField(matrix,15,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
     new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_XTERM,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//,0,0,0,0,20,20);
     oldxterm=getApp()->reg().readStringEntry("PROGS","xterm","xterm -sb ");
     xterm->setText(oldxterm);
 
-    new FXLabel(matrix,_("Console file manager program:"),NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    cfm = new FXTextField(matrix,15,NULL,0,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
-    new FXButton(matrix,_(" Select..."),NULL,this,ID_BROWSE_XTERM,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y);//,0,0,0,0,20,20);
-    oldcfm=getApp()->reg().readStringEntry("PROGS","cfm","xterm -e mc -x");
-    cfm->setText(oldcfm);
-
     // Second tab - Options
     new FXTabItem(tabbook,_("&Options"),NULL);
     FXVerticalFrame *options=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);
-    group=new FXGroupBox(options,"Confirmation",GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-    quit=new FXCheckButton(group,_("Confirm quit"));
+    group=new FXGroupBox(options,_("Options"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+    trashcan=new FXCheckButton(group,_("Use trash can for file delete (safe delete)"));
     del=new FXCheckButton(group,_("Confirm delete"));
     overwrite=new FXCheckButton(group,_("Confirm overwrite"));
     ask=new FXCheckButton(group,_("Always ask before file operation"));
+    quit=new FXCheckButton(group,_("Confirm quit"));
     group=new FXGroupBox(options,_("Warnings"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
 #if defined(linux)
     mount=new FXCheckButton (group,_("Warn when mount points are not responding"));
+	show_mount=new FXCheckButton(group, _("Display mount/umount success messages"));
 #endif
     root_warn=new FXCheckButton (group,_("Warn if running as root"));
 
-    group=new FXGroupBox(options,"Images",GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-    thumbnails=new FXCheckButton (group,_("Display images as thumbnails"));
-
     if(getuid())//running as a user?
         root_warn->disable();
     else
         root_warn->setCheck(warn_if_root);
 
-    quit->setCheck(confirm_quit);
+    trashcan->setCheck(use_trashcan);
+	quit->setCheck(confirm_quit);
     del->setCheck(confirm_del);
     overwrite->setCheck(confirm_overwrite);
     ask->setCheck(ask_before_copy);
 #if defined(linux)
 	mount->setCheck(mount_warn);
+	show_mount->setCheck(mount_messages);
 #endif
-	thumbnails->setCheck(display_thumbnails);
 
     // Third tab - Visual
     new FXTabItem(tabbook,_("Co&lors"),NULL);
@@ -344,7 +335,7 @@ long PreferencesBox::onCmdTextFont(FXObject*,FXSelector,void*)
 long PreferencesBox::onCmdAccept(FXObject* o,FXSelector s,void* p)
 {
     if(oldiconpath != iconpath->getText())
-        getApp()->reg().writeStringEntry("SETTINGS","iconpath",iconpath->getText().text());
+		getApp()->reg().writeStringEntry("SETTINGS","iconpath",iconpath->getText().text());
 
     if(oldeditor != editor->getText())
         getApp()->reg().writeStringEntry("PROGS","editor",editor->getText().text());
@@ -355,28 +346,27 @@ long PreferencesBox::onCmdAccept(FXObject* o,FXSelector s,void* p)
     if(oldxterm != xterm->getText())
         getApp()->reg().writeStringEntry("PROGS","xterm",xterm->getText().text());
 
-    if(oldcfm != cfm->getText())
-        getApp()->reg().writeStringEntry("PROGS","cfm",cfm->getText().text());
-
     confirm_quit=quit->getCheck();
+	use_trashcan=trashcan->getCheck();
     confirm_del=del->getCheck();
     confirm_overwrite=overwrite->getCheck();
     ask_before_copy=ask->getCheck();
     warn_if_root=root_warn->getCheck();
 #if defined(linux)
 	mount_warn=mount->getCheck();
+	mount_messages=show_mount->getCheck();
 #endif
-	display_thumbnails=thumbnails->getCheck();
 	
     getApp()->reg().writeUnsignedEntry("OPTIONS","confirm_quit",quit->getCheck());
+    getApp()->reg().writeUnsignedEntry("OPTIONS","use_trashcan",trashcan->getCheck());
     getApp()->reg().writeUnsignedEntry("OPTIONS","confirm_delete",del->getCheck());
     getApp()->reg().writeUnsignedEntry("OPTIONS","confirm_overwrite",overwrite->getCheck());
     getApp()->reg().writeUnsignedEntry("OPTIONS","ask_before_copy",ask->getCheck());
     getApp()->reg().writeUnsignedEntry("OPTIONS","root_warn",root_warn->getCheck());
 #if defined(linux)
     getApp()->reg().writeUnsignedEntry("OPTIONS","mount_warn",mount->getCheck());
+	getApp()->reg().writeUnsignedEntry("OPTIONS","mount_messages",show_mount->getCheck());
 #endif
-    getApp()->reg().writeUnsignedEntry("OPTIONS","display_thumbnails",thumbnails->getCheck());
 
     if(currTheme != Themes[0])
     {
