@@ -42,24 +42,23 @@ private:
     FXTextField *editor;
     FXTextField *viewer;
     FXTextField *xterm;
-    FXTextField *cfm;
     FXTextField *normalfont;
     FXTextField *textfont;
     FXString    oldiconpath;
     FXString    oldeditor;
     FXString    oldviewer;
     FXString    oldxterm;
-    FXString    oldcfm;
     FXString    oldnormalfont;
     FXString    oldtextfont;
     FXCheckButton  *quit;
+    FXCheckButton  *trashcan;
     FXCheckButton  *del;
     FXCheckButton  *overwrite;
     FXCheckButton  *ask;
     FXCheckButton  *bg;
     FXCheckButton  *root_warn;
     FXCheckButton  *mount;
-    FXCheckButton  *thumbnails;
+	FXCheckButton  *show_mount;
     Theme          Themes[THEMES_NUM];
     Theme          currTheme;
 public:
@@ -68,7 +67,6 @@ public:
         ID_BROWSE_EDIT,
         ID_BROWSE_VIEW,
         ID_BROWSE_XTERM,
-        ID_BROWSE_CFM,
         ID_COLOR,
 		ID_NORMALFONT,
 		ID_TEXTFONT,
