@@ -17,10 +17,11 @@
 #include <mntent.h>
 #endif
 
-#include <fox-1.2/fx.h>
-#include <fox-1.2/fxkeys.h>
-#include <fox-1.2/FXPNGIcon.h>
+#include <fox-1.4/fx.h>
+#include <fox-1.4/fxkeys.h>
+#include <fox-1.4/FXPNGIcon.h>
 
+#include "xfedefs.h"
 #include "icons.h"
 #include "FileDialog.h"
 #include "OverwriteBox.h"
@@ -31,26 +32,13 @@
 #include "MessageBox.h"
 
 
-
-// Maximum file name length
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 1024
-#endif
-
-// Root directory string
-#ifndef ROOTDIR
-#define ROOTDIR "/"
-#endif
-
-
 // Global variables
 extern FXMainWindow *mainWindow;
-extern FXStringDict *devices;
+extern FXStringDict *fsdevices;
 
 
 // Map
-FXDEFMAP(PermFrame) PermFrameMap[]={
-                                   };
+FXDEFMAP(PermFrame) PermFrameMap[]={};
 
 // Object implementation
 FXIMPLEMENT(PermFrame,FXVerticalFrame,PermFrameMap,ARRAYNUMBER(PermFrameMap))
@@ -60,7 +48,9 @@ PermFrame::PermFrame(FXComposite* parent,FXObject *target):
 {
     FXHorizontalFrame *accessframe=new FXHorizontalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y);
     FXHorizontalFrame *chmodframe=new FXHorizontalFrame(this,PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-    FXGroupBox *group1=new FXGroupBox(accessframe,_("User"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X);
+    
+	// Permissions
+	FXGroupBox *group1=new FXGroupBox(accessframe,_("User"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X);
     ur=new FXCheckButton(group1,_("Read"),target,PropertiesBox::ID_RUSR);
     uw=new FXCheckButton(group1,_("Write"),target,PropertiesBox::ID_WUSR);
     ux=new FXCheckButton(group1,_("Execute"),target,PropertiesBox::ID_XUSR);
@@ -73,21 +63,13 @@ PermFrame::PermFrame(FXComposite* parent,FXObject *target):
     ow=new FXCheckButton(group3,_("Write"),target,PropertiesBox::ID_WOTH);
     ox=new FXCheckButton(group3,_("Execute"),target,PropertiesBox::ID_XOTH);
 
-    FXGroupBox *group4=new FXGroupBox(chmodframe,_("Command"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-	radiotarget.connect(cmd);
-	set=new FXRadioButton(group4,_("Set marked"),&radiotarget,FXDataTarget::ID_OPTION+PropertiesBox::ID_SET);
-    clear=new FXRadioButton(group4,_("Clear marked"),&radiotarget,FXDataTarget::ID_OPTION+PropertiesBox::ID_CLEAR);
-    add=new FXRadioButton(group4,_("Add marked"),&radiotarget,FXDataTarget::ID_OPTION+PropertiesBox::ID_ADD);
-    rec=new FXCheckButton(group4,_("Recursive folders"),NULL,0);
-    dironly=new FXCheckButton(group4,_("Folders only"),NULL,0);
-    dironly->disable();
-
-    FXGroupBox *group5=new FXGroupBox(chmodframe,_("Owner"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-    new FXLabel(group5,_("User"));
-    user=new FXComboBox(group5,10,NULL,0,COMBOBOX_NO_REPLACE|FRAME_SUNKEN|FRAME_THICK);
+    // Owner
+	FXGroupBox *group4=new FXGroupBox(chmodframe,_("Owner"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+    new FXLabel(group4,_("User"));
+    user=new FXComboBox(group4,15,NULL,0,COMBOBOX_NO_REPLACE|FRAME_SUNKEN|FRAME_THICK);
   	user->setNumVisible(5);
-    new FXLabel(group5,_("Group"));
-    grp=new FXComboBox(group5,10,NULL,0,COMBOBOX_NO_REPLACE|FRAME_SUNKEN|FRAME_THICK);
+    new FXLabel(group4,_("Group"));
+    grp=new FXComboBox(group4,15,NULL,0,COMBOBOX_NO_REPLACE|FRAME_SUNKEN|FRAME_THICK);
   	grp->setNumVisible(5);
 
 	// User names (sorted in ascending order)
@@ -109,6 +91,18 @@ PermFrame::PermFrame(FXComposite* parent,FXObject *target):
     endgrent();
 	grp->setSortFunc(FXList::ascending);
 	grp->sortItems();	
+
+    // Command
+	FXGroupBox *group5=new FXGroupBox(chmodframe,_("Command"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+	radiotarget.connect(cmd);
+	set=new FXRadioButton(group5,_("Set marked"),&radiotarget,FXDataTarget::ID_OPTION+PropertiesBox::ID_SET);
+    clear=new FXRadioButton(group5,_("Clear marked"),&radiotarget,FXDataTarget::ID_OPTION+PropertiesBox::ID_CLEAR);
+    add=new FXRadioButton(group5,_("Add marked"),&radiotarget,FXDataTarget::ID_OPTION+PropertiesBox::ID_ADD);
+    own=new FXCheckButton(group5,_("Owner only"),NULL,0);
+    rec=new FXCheckButton(group5,_("Recursively"),NULL,0);
+    dironly=new FXCheckButton(group5,_("Folders only"),NULL,0);
+    dironly->disable();
+
 }
 
 // Map
@@ -124,7 +118,7 @@ FXDEFMAP(PropertiesBox) PropertiesBoxMap[]={
             FXMAPFUNC(SEL_COMMAND,PropertiesBox::ID_ROTH,PropertiesBox::onCmdCheck),
             FXMAPFUNC(SEL_COMMAND,PropertiesBox::ID_WOTH,PropertiesBox::onCmdCheck),
             FXMAPFUNC(SEL_COMMAND,PropertiesBox::ID_XOTH,PropertiesBox::onCmdCheck),
-  			FXMAPFUNC(SEL_UPDATE,0,PropertiesBox::onUpdDironly),
+  			FXMAPFUNC(SEL_UPDATE,0,PropertiesBox::onUpdPerm),
             FXMAPFUNC(SEL_COMMAND,PropertiesBox::ID_SET,PropertiesBox::onCmdCommand),
             FXMAPFUNC(SEL_COMMAND,PropertiesBox::ID_CLEAR,PropertiesBox::onCmdCommand),
             FXMAPFUNC(SEL_COMMAND,PropertiesBox::ID_ADD,PropertiesBox::onCmdCommand),
@@ -142,16 +136,17 @@ FXIMPLEMENT(PropertiesBox,FXDialogBox,PropertiesBoxMap,ARRAYNUMBER(PropertiesBox
 // Construct window for one file
 PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialogBox(win,_("Properties"), DECOR_TITLE|DECOR_BORDER/*,0,0,350,200*/)
 {
-    unsigned long long filesize, dsize;
+    unsigned long long filesize, dirsize=0;
     FXString mod,changed,accessed;
-	FXString grpid, usrid;
+	FXString grpid, usrid;	
+	FXLabel *sizelabel=NULL;
     struct stat info;
-    FXString type, extension;
-    int islink = 0;
-	int isdirectory = 0;
+    FXString type, extension, fileassoc;
+    int islink, isdirectory;
     char filename[MAXPATHLEN+1],linkto[MAXPATHLEN+1];
     char mnttype[64], used[64], avail[64], size[64];
     char buf[MAXPATHLEN+1];
+	FXString hsize;
 	FXbool ismountpoint=FALSE;
 	FILE *p;
 
@@ -224,9 +219,7 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 	}
 #if defined(linux)
 	else if(extension=="rpm")
-	{
 		editlbl = _("Install/Upgrade:");
-	}
 #endif
 
     new FXLabel(matrix,viewlbl,NULL,JUSTIFY_LEFT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW);
@@ -256,7 +249,7 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
     input = new FXTextField(generalframe,20,NULL,0,LAYOUT_SIDE_TOP|/*LAYOUT_CENTER_Y|*/LAYOUT_CENTER_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW|LAYOUT_FILL_X);
 
     // Complete filename (with path)
-	strcpy(filename,(path+PATHSEPSTRING+file).text());
+	strlcpy(filename,(path+PATHSEPSTRING+file).text(),(path+PATHSEPSTRING+file).length()+1);
 	parentdir=path;
 	
     // Get file/link info
@@ -289,7 +282,7 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 		if(nread>0)
 		{
 			linkto[nread]='\0';
-			strcpy(filename,linkto);
+			strlcpy(filename,linkto,sizeof(linkto));
 		}
 	}
 	else
@@ -308,16 +301,13 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 
     // Size
     filesize=(unsigned long long)info.st_size;
-    FXString fileassoc;
 
     // Is it a directory?
 	isdirectory=S_ISDIR(info.st_mode);
     if(isdirectory)
-    {
-		
+    {		
 		// Directory path
 		FXString dupath=FXFile::absolute(path,file);
-		dupath=quote(dupath,FALSE);
 
 #if defined(linux)
 		FILE *mtab=setmntent("/etc/mtab","r");
@@ -326,31 +316,34 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
         {
             while((mnt=getmntent(mtab)))
             {
-                if(strcmp(mnt->mnt_type,MNTTYPE_IGNORE) && strcmp(mnt->mnt_type,MNTTYPE_SWAP) && strcmp(mnt->mnt_type,"proc"))
+                if(!::streq(mnt->mnt_type,MNTTYPE_IGNORE) && !::streq(mnt->mnt_type,MNTTYPE_SWAP) && !::streq(mnt->mnt_type,"proc"))
                 {
-                    if(!strcmp(mnt->mnt_dir,dupath.text()))
+                    if(::streq(mnt->mnt_dir,dupath.text()))
                     {
                         ismountpoint=TRUE;
-                        sprintf(buf,_("Filesystem (%s)"),mnt->mnt_fsname);
+                        snprintf(buf,sizeof(buf)-1,_("Filesystem (%s)"),mnt->mnt_fsname);
                         type+=buf;
-						strcpy(mnttype,mnt->mnt_type);
+						strlcpy(mnttype,mnt->mnt_type,strlen(mnt->mnt_type)+1);
                     }
                 }
             }
             endmntent(mtab);
         }
 #endif
-			// If it is a mount point
+		// If it is a mount point
 		if(ismountpoint)
 		{
-			sprintf(buf,"df --block-size=1 %s",filename);
+			snprintf(buf,sizeof(buf)-1,"df --block-size=1 '%s'",filename);
 			p=popen(buf,"r");
 			fgets(buf,sizeof(buf),p);
 			fgets(buf,sizeof(buf),p);
 			strtok(buf," "); // should check if str!=NULL
 			strtok(NULL," ");// get size
-			strcpy(used,strtok(NULL," "));// get used
-			strcpy(avail,strtok(NULL," "));// get available
+			char *pstr;
+			pstr=strtok(NULL," ");
+			strlcpy(used,pstr,strlen(pstr)+1);// get used
+			pstr=strtok(NULL," ");
+			strlcpy(avail,pstr,strlen(pstr)+1);// get available
 			pclose(p);
 		}
 			
@@ -359,9 +352,9 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 		{
             type+=_("Folder");
     		getApp()->beginWaitCursor();
-			strcpy(buf,dupath.text());
-			dsize=dirpath(buf);
-			sprintf(size,"%llu",dsize);
+			strlcpy(buf,dupath.text(),dupath.length()+1);
+			dirsize=dirpath(buf);
+			snprintf(size,sizeof(size)-1,"%llu",dirsize);
     		getApp()->endWaitCursor();
 		}
     }
@@ -468,7 +461,7 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
   		FXMatrix *attrmatrix=new FXMatrix(attrgroup,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y);
   		new FXLabel(attrmatrix,_("Type:"),NULL,LAYOUT_LEFT);
   		fileType=new FXLabel(attrmatrix,NULL,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
-  		new FXLabel(attrmatrix,_("Size:"),NULL,LAYOUT_LEFT);
+  		sizelabel=new FXLabel(attrmatrix,_("Total size:"),NULL,LAYOUT_LEFT);
   		fileSize=new FXLabel(attrmatrix,NULL,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
 
   		FXGroupBox *timegroup=new FXGroupBox(generalframe,_("File Time"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X);
@@ -490,11 +483,11 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 		// if mount point
 		if(ismountpoint)
 		{
-			::hSize(used,used);
-			::hSize(avail,avail);
+			hsize=::hSize(used);
+			mtUsed->setText(hsize);
+			hsize=::hSize(avail);
+			mtFree->setText(hsize);
 			mtType->setText(mnttype);
-			mtUsed->setText(used);
-			mtFree->setText(avail);
 		}
 		// if folder
 		else
@@ -502,17 +495,18 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
         	fileModified->setText(mod);
         	fileChanged->setText(changed);
         	fileAccessed->setText(accessed);
-			::hSize(size,size);
+			hsize=::hSize(size);
+        	snprintf(size,sizeof(size)-1,_("%s (%llu bytes)"),hsize.text(),dirsize);        
 			fileSize->setText(size);
 		}
 	}
 	// Regular file
     else
     {
-        FXchar size[64], hsize[64];
-        sprintf(size,"%llu",filesize);
-        ::hSize(size,hsize);
-        sprintf(size,"%s (%llu bytes)",hsize,filesize);        
+        snprintf(size,sizeof(size)-1,"%llu",filesize);
+        hsize=::hSize(size);
+        snprintf(size,sizeof(size)-1,_("%s (%llu bytes)"),hsize.text(),filesize);        
+		sizelabel->setText(_("Size:"));
 		fileSize->setText(size);
         fileModified->setText(mod);
         fileChanged->setText(changed);
@@ -526,10 +520,15 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString file,FXString path): FXDialo
 }
 
 // Construct window for multiple files
-PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n): FXDialogBox(win,_("Properties"), DECOR_TITLE|DECOR_BORDER/*,0,0,350,200*/)
+PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n,FXString path): FXDialogBox(win,_("Properties"), DECOR_TITLE|DECOR_BORDER/*,0,0,350,200*/)
 {
+	unsigned long long filesize, dirsize, totalsize=0;
 	struct stat info;
 	FXString grpid, usrid;
+    FXString type, extension, fileassoc;
+    char buf[MAXPATHLEN+1];
+	int i, isdirectory, nbdirs=0, nbfiles=0, dotdot=0;
+	FXbool firstfile=TRUE;
 
     // Buttons
     FXHorizontalFrame *buttons=new FXHorizontalFrame(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X,0,0,0,0,10,10,5,5);
@@ -547,14 +546,25 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n): FXDialogBox(wi
     // Switcher
     FXTabBook *tabbook = new FXTabBook(contents,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT);
 
+    // First item is General
+    new FXTabItem(tabbook,_("&General"),NULL);
+    FXVerticalFrame *generalframe=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);
+	FXGroupBox *attrgroup=new FXGroupBox(generalframe,_("Properties"),GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+	FXMatrix *attrmatrix=new FXMatrix(attrgroup,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y);
+	new FXLabel(attrmatrix,_("Selection:"),NULL,LAYOUT_LEFT);
+	FXLabel *filesSelected=new FXLabel(attrmatrix,NULL,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
+	new FXLabel(attrmatrix,_("Type:"),NULL,LAYOUT_LEFT);
+	FXLabel *filesType=new FXLabel(attrmatrix,NULL,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
+	new FXLabel(attrmatrix,_("Total size:"),NULL,LAYOUT_LEFT);
+	FXLabel *filesSize=new FXLabel(attrmatrix,NULL,NULL,LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
 
-    // Here, access Permissions is the only item shown
+    // Second item is Access Permissions
     new FXTabItem(tabbook,_("&Permissions"),NULL);
     perm = new PermFrame(tabbook,this);
 
-    // Get file/link info of the first file of the list
+    // Get file/link info of the second file of the list (the first should be ".."
 	// This is used as a guess for the username, group and permissions of the whole list
-    if(lstatout(file[0].text(),&info)!=0)
+    if(lstatout(file[1].text(),&info)!=0)
         return ;
 
 	// Obtain user name
@@ -582,10 +592,109 @@ PropertiesBox::PropertiesBox(FXWindow *win,FXString *file,int n): FXDialogBox(wi
     perm->grp->setText(grpid);
 
     mode = orig_mode;
-    perm->cmd = PropertiesBox::ID_ADD;
+    perm->cmd = PropertiesBox::ID_SET;
     files = file;
     source = "";
     num = n;
+
+	// Start wait cursor
+	getApp()->beginWaitCursor();
+
+	// Total size and files type
+	for(i=0;i<num;i++)
+	{
+		if(lstatout(files[i].text(),&info)!=0)
+			continue ;
+
+		// Special case of the ".." directory
+		if (files[i]=="..")
+		{	
+			dotdot=1;
+			continue;
+		}
+
+		// Is it a directory?
+		isdirectory=S_ISDIR(info.st_mode);
+		if(isdirectory)
+		{
+			// Recursive size
+			FXString dupath=FXFile::absolute(path,files[i]);
+			strlcpy(buf,dupath.text(),dupath.length()+1);
+			dirsize=dirpath(buf);
+			totalsize+=dirsize;
+			nbdirs++;
+		}
+		else // regular file
+		{
+			// File size	
+    		filesize=(unsigned long long)info.st_size;
+			totalsize+=filesize;
+			nbfiles++;
+
+			// Try to use association table	
+			extension=FXFile::name(files[i]).rafter('.',1);
+       		if (extension != "")
+				fileassoc=getApp()->reg().readStringEntry("FILETYPES",extension.text(),"");
+
+			// Keep the first encountered type
+			if (firstfile)
+			{
+				// If we have an association
+        		if(!fileassoc.empty())
+ 					type=fileassoc.section(';',1);
+				else
+					type=_("Document");
+				firstfile=FALSE;
+			}
+			else
+			{
+				// If we have an association
+        		if (!fileassoc.empty())
+				{
+					if (fileassoc.section(';',1)!=type)
+						type=_("Multiple types");
+				}
+				else
+				{
+					if (type!=_("Document"))
+						type=_("Multiple types");
+				}
+			}
+		}
+	}
+
+	// Special cases of the file type
+	if (nbfiles==0)
+		type=_("Folder");
+	
+	if (nbdirs>=1 && nbfiles>=1)
+		type=_("Multiple types");
+	
+	// Display total size
+	FXchar size[64];
+	FXString hsize;
+	snprintf(size,sizeof(size)-1,"%llu",totalsize);
+	hsize=::hSize(size);
+	snprintf(size,sizeof(size)-1,_("%s (%llu bytes)"),hsize.text(),totalsize);        
+	filesSize->setText(size);
+	
+	// Number of selected files
+	snprintf(buf,sizeof(buf)-1,_("%d items in %d folder(s) and %d file(s)"),num,nbdirs+dotdot,nbfiles);
+	filesSelected->setText(buf);
+
+	// Display type of selected files
+	filesType->setText(type);
+
+	// Stop wait cursor
+	getApp()->endWaitCursor();
+}
+
+
+// Make window
+void PropertiesBox::create()
+{
+    FXDialogBox::create();
+    show(PLACEMENT_SCREEN);
 }
 
 
@@ -598,8 +707,12 @@ long PropertiesBox::onCmdAcceptMult(FXObject* o,FXSelector s,void* p)
     FXDialogBox::onCmdAccept(o,s,p);
     if(mode)
     {
-		f = new File(getApp(),_("File permissions"),CHMOD);
+		f = new File(this,_("File permissions"),CHMOD);
 		f->create();
+
+		// Wait cursor
+		getApp()->beginWaitCursor();
+
         for(i=0;i<num;i++)
         {
             struct stat info;
@@ -619,10 +732,11 @@ long PropertiesBox::onCmdAcceptMult(FXObject* o,FXSelector s,void* p)
                 	m=mode;
                 	break;
             	default :
-                	return 1;
+					getApp()->endWaitCursor();
+					return 1;
             }
             
-			if (files[i]!="..")
+			if (files[i]!=".." && !perm->own->getCheck())
 			{
 				rc=f->chmod(strdup(files[i].text()),file,m,perm->rec->getCheck(),perm->dironly->getCheck());
 				
@@ -632,6 +746,7 @@ long PropertiesBox::onCmdAcceptMult(FXObject* o,FXSelector s,void* p)
 					f->hide();
 					MessageBox::error(this,MBOX_OK,_("Error"),_("Change file(s) permissions cancelled!"));
 					delete f;
+					getApp()->endWaitCursor();
 					return 1;
 				}
         		
@@ -644,14 +759,19 @@ long PropertiesBox::onCmdAcceptMult(FXObject* o,FXSelector s,void* p)
 			}
         }
 		delete f;
+		getApp()->endWaitCursor();
     }
 
     rc = 0;
     // chown if needed
     if(!perm->grp->getText().empty() || !perm->user->getText().empty())
     {
-        f = new File(getApp(),_("File owner"),CHOWN);
+        f = new File(this,_("File owner"),CHOWN);
         f->create();
+
+		// Wait cursor
+		getApp()->beginWaitCursor();
+
         for(i=0;i<num;i++)
         {
             struct stat info;
@@ -681,6 +801,7 @@ long PropertiesBox::onCmdAcceptMult(FXObject* o,FXSelector s,void* p)
 				f->hide();
 				MessageBox::error(this,MBOX_OK,_("Error"),_("Change owner cancelled!"));
 				delete f;
+				getApp()->endWaitCursor();
 				return 1;
 			}
 			
@@ -692,7 +813,10 @@ long PropertiesBox::onCmdAcceptMult(FXObject* o,FXSelector s,void* p)
             }
         }
 		delete f;
+		getApp()->endWaitCursor();
     }
+	// FilePanel and DirPanel refresh
+	//mainWindow->handle(this,FXSEL(SEL_COMMAND,XFileExplorer::ID_REFRESH),NULL);
     return 1;
 }
 
@@ -706,7 +830,7 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
     FXString oldfileassoc,fileassoc,op,v,e;
 
 	// Don't proceed any task on the parent directory
-	if (!strcmp(source.text(),".."))
+	if (::streq(source.text(),".."))
 		return 1;
 
 	op = open->getText();
@@ -738,8 +862,8 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
 			str=new char*[2];
 			str[0]=new char[strlen(ext->getText().text())+1];
 			str[1]=new char[strlen(command.text())+1];
-			strcpy(str[0],ext->getText().text());
-			strcpy(str[1],command.text());
+			strlcpy(str[0],ext->getText().text(),ext->getText().length()+1);
+			strlcpy(str[1],command.text(),command.length()+1);
 			mainWindow->handle(this,FXSEL(SEL_COMMAND,XFileExplorer::ID_FILE_ASSOC),str);
     	}
 	}
@@ -748,19 +872,19 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
 	if(source != target)
 	{
 		// Source is not writable
-		if (!FXFile::isWritable(source))
+		if (!::isWritable(source))
         	MessageBox::error(this,MBOX_OK,_("Error"),_("Can't write to %s : Permission denied"),source.text());
 		
 		// Target parent directory doesn't exist or is not writable
 		FXString targetparentdir=FXFile::directory(target);
 		if (targetparentdir=="")
 			targetparentdir=parentdir;
-		if (!exists(dequote(targetparentdir)))
+		if (!::exists(targetparentdir))
         {
 			MessageBox::error(this,MBOX_OK,_("Error"),_("Folder %s doesn't exist"),targetparentdir.text());
         	return 0;
 		}
-		if (!FXFile::isWritable(targetparentdir))
+		if (!::isWritable(targetparentdir))
 		{
          	MessageBox::error(this,MBOX_OK,_("Error"),_("Can't write to %s : Permission denied"),targetparentdir.text());
         	return 0;
@@ -770,7 +894,7 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
 		else
 		{
 			File *f;
-			f=new File(getApp(),_("File rename"),RENAME);
+			f=new File(this,_("File rename"),RENAME);
        		f->create();
 			f->rename(source,input->getText());
 			delete f;
@@ -784,15 +908,18 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
 		input->setText(source);
 		return 1;
 	}
-    if(!(orig_mode == mode && perm->cmd == PropertiesBox::ID_SET && !perm->rec->getCheck()))//no change
+    // Chmod only if "Owner only" is not checked
+	if (!perm->own->getCheck())
     {
-        f = new File(getApp(),_("File permissions"),CHMOD);
-        f->create();
-
         struct stat info;
         mode_t m;
-        if(lstatout(input->getText().text(),&info)!=0)
+		
+		FXString filepath=parentdir+"/"+input->getText().text();
+        if(lstatout(filepath.text(),&info)!=0)
             return 1;
+			
+        f = new File(this,_("File permissions"),CHMOD);
+        f->create();
 
         switch(perm->cmd)
         {
@@ -808,8 +935,12 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
         	default :
             	return 1;
         }
+
+		// Wait cursor
+		getApp()->beginWaitCursor();
+
 		// Perform chmod on the selected file or directory
-		rc=f->chmod(strdup(input->getText().text()),file,m,perm->rec->getCheck(),perm->dironly->getCheck());
+		rc=f->chmod(strdup(filepath.text()),file,m,perm->rec->getCheck(),perm->dironly->getCheck());
 
 		// If action is cancelled in progress dialog
 		if (f->isCancelled)
@@ -817,9 +948,11 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
 			f->hide();
 			MessageBox::error(this,MBOX_OK,_("Error"),_("Change file permissions cancelled!"));
 			delete f;
+			getApp()->endWaitCursor();
 			return 1;
 		}
 		delete f;
+		getApp()->endWaitCursor();
         
 		// Handle chmod errors
 		if(rc)
@@ -828,9 +961,9 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
 
     // chown if needed
     rc = 0;
-    if(oldgrp != perm->grp->getText() || oldusr != perm->user->getText())
+    if (oldgrp != perm->grp->getText() || oldusr != perm->user->getText() || perm->rec->getCheck())
     {
-        f = new File(getApp(),_("File owner"),CHOWN);
+        f = new File(this,_("File owner"),CHOWN);
         f->create();
 
         uid_t uid = 32768;
@@ -847,8 +980,12 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
                 gid = grpe->gr_gid;
         endgrent();
 
+		// Wait cursor
+		getApp()->beginWaitCursor();
+
         // Perform chown on the selected file or directory
-		rc = f->chown(strdup(input->getText().text()),file,uid,gid,perm->rec->getCheck(),perm->dironly->getCheck());
+		FXString filepath=parentdir+"/"+input->getText().text();
+		rc = f->chown(strdup(filepath.text()),file,uid,gid,perm->rec->getCheck(),perm->dironly->getCheck());
 
 		// If action is cancelled in progress dialog
 		if (f->isCancelled)
@@ -856,14 +993,19 @@ long PropertiesBox::onCmdAcceptSingle(FXObject* o,FXSelector s,void* p)
 			f->hide();
 			MessageBox::error(this,MBOX_OK,_("Error"),_("Change owner cancelled!"));
 			delete f;
+			getApp()->endWaitCursor();
 			return 1;
 		}
 		delete f;
+		getApp()->endWaitCursor();
 		
 		// Handle chown errors
         if(rc)
             MessageBox::error(this,MBOX_OK,_("Error"),_("Chown failed in %s: %s"),file,strerror(errno));
     }
+	
+	// FilePanel and DirPanel refresh
+	//mainWindow->handle(this,FXSEL(SEL_COMMAND,XFileExplorer::ID_REFRESH),NULL);
     return 1;
 }
 
@@ -967,7 +1109,7 @@ long PropertiesBox::onCmdBrowseIcon(FXObject* o,FXSelector s,void* p)
     if(browseIcon.execute())
     {
         char *path=strdup(browseIcon.getFilename().text());
-        if(!exists(dequote(path)))
+        if(!::exists(path))
             return 0;
         if(FXSELID(s)==ID_BIG_ICON)
             bigic->setText(path);
@@ -977,16 +1119,48 @@ long PropertiesBox::onCmdBrowseIcon(FXObject* o,FXSelector s,void* p)
     return 1;
 }
 
-long PropertiesBox::onUpdDironly(FXObject* o,FXSelector s,void* p)
+long PropertiesBox::onUpdPerm(FXObject* o,FXSelector s,void* p)
 {
     if(perm->rec->getCheck())
         perm->dironly->enable();
     else
         perm->dironly->disable();
-    return 1;
 
+    if(perm->own->getCheck())
+	{
+        perm->set->disable();
+        perm->clear->disable();
+        perm->add->disable();
+		perm->ur->disable();
+		perm->uw->disable();
+		perm->ux->disable();
+		perm->gr->disable();
+		perm->gw->disable();
+		perm->gx->disable();
+		perm->or_->disable();
+		perm->ow->disable();
+		perm->ox->disable();
+	}
+    else
+	{
+        perm->set->enable();
+        perm->clear->enable();
+        perm->add->enable();
+		perm->ur->enable();
+		perm->uw->enable();
+		perm->ux->enable();
+		perm->gr->enable();
+		perm->gw->enable();
+		perm->gx->enable();
+		perm->or_->enable();
+		perm->ow->enable();
+		perm->ox->enable();
+	}
+
+    return 1;
 }
 
+
 long PropertiesBox::onCmdKeyPress(FXObject* sender,FXSelector sel,void* ptr)
 {
     FXEvent* event=(FXEvent*)ptr;
