@@ -5,7 +5,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <fox-1.2/fx.h>
+#include <fox-1.4/fx.h>
 
 #include "TextWindow.h"
 #include "icons.h"
@@ -21,7 +21,7 @@ FXIMPLEMENT(TextWindow,FXDialogBox,TextWindowMap,ARRAYNUMBER(TextWindowMap))
 
 // Construct Text dialog box
 TextWindow::TextWindow(FXWindow *owner, const FXString& name, FXint nblines, FXint nbcols):
-        FXDialogBox(owner,name,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0, 6,6,6,6, 4,4)
+        FXDialogBox(owner,name,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_MAXIMIZE,0,0,0,0, 6,6,6,6, 4,4)
 {
 
     // Bottom part
@@ -51,6 +51,13 @@ void TextWindow::appendText(const FXchar* str)
 	getApp()->repaint();
 }
 
+// Change the text in the buffer to new text
+void TextWindow::setFont(FXFont* font)
+{
+	text->setFont(font);
+	getApp()->repaint();
+}
+
 // Scroll to the last line
 void TextWindow::scrollToLastLine(void)
 {
